Package: HCATonsilData
Title: Provide programmatic access to the tonsil cell atlas datasets
Version: 1.9.0
Authors@R: c(person("Ramon", "Massoni-Badosa", role = c("aut", "cre"),
                    email ="ramonmassoni@gmail.com",
                    comment = c(ORCID = "0000-0001-7115-8145")),
             person("Federico", "Marini", role = c("aut"),
                    email = "marinif@uni-mainz.de", 
                    comment = c(ORCID = "0000-0003-3252-7758")),
             person("Alan", "O'Callaghan", role = c("aut"),
                    email = "alan.ocallaghan@outlook.com"),
	           person("Helena L.", "Crowell", role=c("aut"),
	                  email="helena@crowell.eu", 
	                  comment = c(ORCID="0000-0002-4801-1767"))
             )
Description: This package provides access to the scRNA-seq, scATAC-seq,
    multiome, CITE-seq and spatial transcriptomics (Visium) data generated
    by the tonsil cell atlas in the context of the Human Cell Atlas (HCA).
    The data is provided via the Bioconductor project in the form of 
    SingleCellExperiments. Additionally, information on the whole compendium of
    identified cell types is provided in form of a glossary.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
Imports: ExperimentHub, SingleCellExperiment, SpatialExperiment,
        HDF5Array, SummarizedExperiment, S4Vectors, htmltools,
        rmarkdown, base64enc, utils
Suggests: knitr, ggplot2, testthat (>= 3.0.0), scater, Seurat, Signac,
        zellkonverter, iSEE, ggspavis, kableExtra, BiocStyle
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/massonix/HCATonsilData
BugReports: https://github.com/massonix/HCATonsilData/issues
biocViews: SingleCellData, ExperimentData, RNASeqData, ExperimentHub,
        ExpressionData, SpatialData
Depends: R (>= 4.3.0)
git_url: https://git.bioconductor.org/packages/HCATonsilData
git_branch: devel
git_last_commit: 3cdd05d
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 16:24:14 UTC; biocbuild
Author: Ramon Massoni-Badosa [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7115-8145>),
  Federico Marini [aut] (ORCID: <https://orcid.org/0000-0003-3252-7758>),
  Alan O'Callaghan [aut],
  Helena L. Crowell [aut] (ORCID:
    <https://orcid.org/0000-0002-4801-1767>)
Maintainer: Ramon Massoni-Badosa <ramonmassoni@gmail.com>

# Generated by roxygen2: do not edit by hand

export(HCATonsilData)
export(TonsilData_cellinfo)
export(TonsilData_cellinfo_html)
export(TonsilData_glossary)
export(listCellTypes)
export(updateAnnotation)
importFrom(ExperimentHub,ExperimentHub)
importFrom(HDF5Array,HDF5Array)
importFrom(S4Vectors,DataFrame)
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDims)
importFrom(SpatialExperiment,SpatialExperiment)
importFrom(SpatialExperiment,SpatialImage)
importFrom(SummarizedExperiment,assay)
importFrom(base64enc,dataURI)
importFrom(htmltools,tags)
importFrom(rmarkdown,render)
importFrom(utils,browseURL)
importFrom(utils,read.table)

# HCATonsilData

<!-- badges: start -->
[![R-CMD-check](https://github.com/massonix/HCATonsilData/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/massonix/HCATonsilData/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

HCATonsilData is an R/ExperimentHub package that provides easy access to
single-cell RNA-seq (scRNA-seq), single-cell ATAC-seq (scATAC-seq), 10X Multiome,
CITE-seq and spatial transcriptomics data (Visium) derived from the tonsil cell
atlas project.


## Citation

The preprint was published in June 2022:

[Massoni-Badosa, R., Soler-Vila, P., Aguilar-Fernández, S., Nieto, J. C., Elosua-Bayes, M., Marchese, D., ... & Heyn, H. (2022). An Atlas of Cells in the Human Tonsil. bioRxiv, 2022-06.](https://www.biorxiv.org/content/10.1101/2022.06.24.497299v1.abstract)

And the final paper was accepted in September 2023, so we expect it to be published
December of this year.

![](inst/images/graphical_abstract.png)



## Installation

HCATonsilData is available in BioConductor and can be installed as follows:

``` r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("HCATonsilData")
```

Alternatively, you can install it from GitHub using the [devtools](https://github.com/hadley/devtools) package:

``` r
if (!require("devtools", quietly = TRUE))
    install.packages("devtools")

devtools::install_github("massonix/HCATonsilData", build_vignettes = TRUE)
```

## Documentation

HCATonsilData is a data package. As such, comprehensive documentation is an essential
component of the package, and we provide it through the package vignette:

``` {r}
browseVignettes("HCATonsilData")
```

## Requirements

HCATonsilData has two versions: version 1.0 corresponds to the first data
release that came out during the preprint. During the revision process, we
included 7 additional tonsils, and almost doubled the number of cells of the
atlas. We provide this data in version 2.0, which is the one associated with
the final publication.

The data for version 2.0 was uploaded to ExperimentHub just before the 3.18
release of BioConductor. Thus, you will need the release >= 3.18 of Bioconductor,
and R >= 4.3.0


## Basic usage

We provide access to `SingleCellExperiment` objects of the main cellular compartments
described in our manuscript. The function `listCellTypes` prints the available
cell types for a given assay (RNA, ATAC, CITE or Spatial):

``` {r}
listCellTypes(assayType = "RNA")
```
```
 [1] "All"        "NBC-MBC"    "GCBC"      
 [4] "PC"         "CD4-T"      "Cytotoxic" 
 [7] "myeloid"    "FDC"        "epithelial"
[10] "PDC" 
```

HCATonsilAtlas data allows to install scRNA-seq and spatial transcriptomics data
as SingleCellExperiments and SpatialExperiments, respectively. For scATAC-seq, 
Multiome, and CITE-seq we provide in the vignette the instructions on how to access
the data archived at [Zenodo](https://zenodo.org/record/8373756) 
Note that for now HCATonsilAtlas only provides access to RNA data, but we are
working hard to extend it to other modalities, which are/will be archived in 
[Zenodo](https://doi.org/10.5281/zenodo.6340174) as [Seurat objects](https://satijalab.org/seurat/).

To obtain the `SingleCellExperiment` object associated with a given cell type we use
the `HCATonsilData()` function:

``` {r}
(myeloid <- HCATonsilData(assayType = "RNA", cellType = "myeloid", version = "2.0"))
```
```
class: SingleCellExperiment 
dim: 37378 5334 
metadata(0):
assays(2): counts logcounts
rownames(37378): AL627309.1 AL627309.3 ... AC136616.1 AC023491.2
rowData names(3): gene_name highly_variable gene_id
colnames(5334): bw94nf57_vm85woki_AAAGTGACAAGGAGTC-1 bw94nf57_vm85woki_AAGCATCCACTAGTAC-1 ...
  zoxefjul_5m23u91h_TTAGCCTGTTCGGGAT-1 zoxefjul_5m23u91h_TTATTGCTCCTTAATC-1
colData names(39): barcode donor_id ... UMAP_1_20230508 UMAP_2_20230508
reducedDimNames(3): PCA UMAP HARMONY
mainExpName: NULL
altExpNames(0):
```
```{r}
library(patchwork)
p1 <- scater::plotUMAP(myeloid, colour_by = "annotation_20230508")
p2 <- scater::plotUMAP(myeloid, colour_by = "SELENOP")
p1 | p2
```
![](inst/images/umaps_myeloid.png)


10X Visium data can be installed as follows:

```{r}
(spe <- HCATonsilData("Spatial"))
```
```
class: SpatialExperiment 
dim: 26846 16224 
metadata(0):
assays(2): counts logcounts
rownames(26846): AL627309.1 AL627309.5 ... AC007325.4 AC007325.2
rowData names(8): gene_name vst.mean ... gene_id highly_variable
colnames: NULL
colData names(24): barcode donor_id ... area sample_id
reducedDimNames(3): HARMONY PCA UMAP
mainExpName: Spatial
altExpNames(0):
spatialCoords names(0) :
imgData names(4): sample_id image_id data scaleFactor
```

To plot gene expression you can use the ggspavis package:

```
library(ggspavis)
sub <- spe[, spe$sample_id == "esvq52_nluss5"]
plt <- plotVisium(sub, fill="SELENOP") + 
  scale_fill_gradientn(colors=rev(hcl.colors(9, "Spectral")))
plt$layers[[2]]$aes_params$size <- 1.5
plt$layers[[2]]$aes_params$alpha <- 1
plt$layers[[2]]$aes_params$stroke <- NA
plt
```

## Annotations

To allow users to traceback the rationale behind each and every of our annotations,
we provide a detailed glossary of 121 cell types and states and related functions
to get the explanation, markers and references of every annotation. You can acces
the glossary as a dataframe as follows:


```{r}
glossary_df <- TonsilData_glossary()
```

To get the glossary for each cell type with nice printing formatting you can use
the `TonsilData_cellinfo()` function:

```{r}
TonsilData_cellinfo("Tfr")
```
```
Tfr
------------------------------
Annotation Level 1: CD4_T
Cell Markers: T-follicular regulatory cells in the tonsils are CD25-. These cells down-regulate effector Treg markers (IL2RA, FOXP3, CTLA4). This cluster expressed high levels of FCRL3, CLNK, LEF1, TCF7, RBMS3, SESN3, and PDE3B. The top marker FCRL3 can bind secretory IgA to suppress the Tfr inhibitory function. TCF7 and LEF1 are essential for Tfr development in mice (Wing et al., 2017; Agarwal et al., 2020 ; Yang et al., 2019).
Cell Markers: FCRL3,CLNK,LEF1,TCF7,RBMS3,SESN3,PDE3B
Related references: Wing2017|10.1073/pnas.1705551114;Agarwal2020|10.1016/j.celrep.2019.12.099;Yang2019|10.1016/j.celrep.2019.05.061
```

Alternatively, you can get a static html with links to markers and articles with
`TonsilData_cellinfo_html`

```
TonsilData_cellinfo_html("Tfr")
```

![](vignettes/tfr_glossary.png)

Although we have put massive effort in annotating tonsillar cell types, cell type
annotations are dynamic by nature. New literature or other interpretations of the
data can challenge and refine our annotations. To accommodate this, we have developed
the `updateAnnotation` function, which allows to periodically provide newer
annotations as extra columns in the `colData` slot of the `SingleCellExperiment`
objects. If you want to contribute in one of these versions of the upcoming annotations,
please [open an issue](https://github.com/massonix/HCATonsilData/issues/new) and
describe your annotation.



## Interoperability

While we provide data in the form of SingleCellExperiment objects, you may want to
analyze your data using a different single-cell data container. In future releases,
we will strive to make the SingleCellExperiment objects compatible with [Seurat v5](https://github.com/satijalab/seurat),
which will come out after this release of BioConductor. Alternatively, you may
want to obtain [AnnData](https://anndata.readthedocs.io/en/latest/) objects to analyze your data in [scanpy](https://scanpy.readthedocs.io/en/stable/installation.html) ecosystem. You
can convert and save the data as follows:

```{r}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("zellkonverter")
library(zellkonverter)
epithelial <- HCATonsilData(assayType = "RNA", cellType = "epithelial")
writeH5AD(sce = epithelial, file = "epithelial.h5ad")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = TRUE
)

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("HCATonsilData")

## ----message=FALSE------------------------------------------------------------
library("HCATonsilData")
library("SingleCellExperiment")
library("ExperimentHub")
library("scater")
library("ggplot2")
library("knitr")
library("kableExtra")
library("htmltools")

## -----------------------------------------------------------------------------
data("donor_metadata")

kable(
  donor_metadata,
  format = "markdown",
  caption = "Donor Metadata",
  align = "c"
) |> kable_styling(full_width = FALSE)

## ----echo=FALSE, out.width = "100%"-------------------------------------------
knitr::include_graphics("tonsil_atlas_cohort.png")

## ----echo=FALSE, out.width = "100%"-------------------------------------------
knitr::include_graphics("tonsil_atlas_n_detected_genes.png")

## ----echo=FALSE, out.width = "100%"-------------------------------------------
knitr::include_graphics("tonsil_atlas_umap_9_compartments.png")

## ----echo=FALSE, out.width = "100%"-------------------------------------------
knitr::include_graphics("tonsil_atlas_umaps.png")

## ----echo=FALSE, out.width = "100%"-------------------------------------------
knitr::include_graphics("tonsil_atlas_umap_discovery_validation_cohort.png")

## ----echo=FALSE, out.width = "100%"-------------------------------------------
knitr::include_graphics("tonsil_atlas_umap_discovery_validation_cohort_CD4.png")

## ----eval=FALSE---------------------------------------------------------------
# (sce <- HCATonsilData(assayType = "RNA", cellType = "All"))
# table(sce$assay)

## -----------------------------------------------------------------------------
listCellTypes(assayType = "RNA")
(epithelial <- HCATonsilData(assayType = "RNA", cellType = "epithelial"))
data("colors_20230508")
scater::plotUMAP(epithelial, colour_by = "annotation_20230508") +
  ggplot2::scale_color_manual(values = colors_20230508$epithelial) +
  ggplot2::theme(legend.title = ggplot2::element_blank())

## -----------------------------------------------------------------------------
data("annotations_dictionary")
annotations_dictionary[["dict_20220619_to_20230508"]]

# load also a predefined palette of colors, to match the ones used in the manuscript 
data("colors_20230508")

(epithelial_discovery <- HCATonsilData("RNA", "epithelial", version = "1.0"))
scater::plotUMAP(epithelial, colour_by = "annotation_20230508") +
  ggplot2::scale_color_manual(values = colors_20230508$epithelial) +
  ggplot2::theme(legend.title = ggplot2::element_blank())

## ----eval=FALSE---------------------------------------------------------------
# library("Seurat")
# library("Signac")
# 
# download_dir = tempdir()
# 
# options(timeout = 10000000)
# atac_url <- "https://zenodo.org/record/8373756/files/TonsilAtlasSeuratATAC.tar.gz"
# download.file(
#   url = atac_url,
#   destfile = file.path(download_dir, "TonsilAtlasSeuratATAC.tar.gz")
# )
# # Advice: check that the md5sum is the same as the one in Zenodo
# untar(
#   tarfile = file.path(download_dir, "TonsilAtlasSeuratATAC.tar.gz"),
#   exdir = download_dir
# )
# atac_seurat <- readRDS(
#   file.path(download_dir, "scATAC-seq/20230911_tonsil_atlas_atac_seurat_obj.rds")
# )
# atac_seurat

## ----eval=FALSE---------------------------------------------------------------
# library("Seurat")
# library("Signac")
# 
# download_dir = tempdir()
# 
# options(timeout = 10000000)
# multiome_url <- "https://zenodo.org/record/8373756/files/TonsilAtlasSeuratMultiome.tar.gz"
# download.file(
#   url = multiome_url,
#   destfile = file.path(download_dir, "TonsilAtlasSeuratMultiome.tar.gz")
# )
# # Advice: check that the md5sum is the same as the one in Zenodo
# untar(
#   tarfile = file.path(download_dir, "TonsilAtlasSeuratMultiome.tar.gz"),
#   exdir = download_dir
# )
# multiome_seurat <- readRDS(
#   file.path(download_dir, "/multiome/20230911_tonsil_atlas_multiome_seurat_obj.rds")
# )
# multiome_seurat

## ----eval=FALSE---------------------------------------------------------------
# download_dir = tempdir()
# 
# fragments_url <- "https://zenodo.org/record/8373756/files/fragments_files.tar.gz"
# download.file(
#   url = fragments_url,
#   destfile = file.path(download_dir, "fragments_files.tar.gz")
# )
# untar(
#   tarfile = file.path(download_dir, "fragments_files.tar.gz"),
#   exdir = download_dir
# )

## ----eval=FALSE---------------------------------------------------------------
# library("Seurat")
# 
# download_dir = tempdir()
# 
# options(timeout = 10000000)
# cite_url <- "https://zenodo.org/record/8373756/files/TonsilAtlasSeuratCITE.tar.gz"
# download.file(
#   url = cite_url,
#   destfile = file.path(download_dir, "TonsilAtlasSeuratCITE.tar.gz")
# )
# # Advice: check that the md5sum is the same as the one in Zenodo
# untar(
#   tarfile = file.path(download_dir, "TonsilAtlasSeuratCITE.tar.gz"),
#   exdir = download_dir
# )
# 
# cite_seurat <- readRDS(
#   file.path(download_dir, "CITE-seq/20220215_tonsil_atlas_cite_seurat_obj.rds")
# )
# cite_seurat

## ----eval=FALSE---------------------------------------------------------------
# scirpy_df <- read.csv(
#   file = file.path(download_dir, "CITE-seq/scirpy_tcr_output.tsv"),
#   header = TRUE
# )
# 
# head(scirpy_df)

## ----eval=FALSE---------------------------------------------------------------
# library("SpatialExperiment")
# (spe <- HCATonsilData(assayType = "Spatial"))

## ----eval=FALSE---------------------------------------------------------------
# library("ggspavis")
# sub <- spe[, spe$sample_id == "esvq52_nluss5"]
# plt <- plotVisium(sub, fill="SELENOP") +
#   scale_fill_gradientn(colors=rev(hcl.colors(9, "Spectral")))
# plt$layers[[2]]$aes_params$size <- 1.5
# plt$layers[[2]]$aes_params$alpha <- 1
# plt$layers[[2]]$aes_params$stroke <- NA
# plt

## -----------------------------------------------------------------------------
glossary_df <- TonsilData_glossary()
head(glossary_df)

## -----------------------------------------------------------------------------
TonsilData_cellinfo("Tfr")

## ----eval = TRUE--------------------------------------------------------------
htmltools::includeHTML(
  TonsilData_cellinfo_html("Tfr", display_plot = TRUE)
)

## ----eval=FALSE---------------------------------------------------------------
# library("zellkonverter")
# epithelial <- HCATonsilData(assayType = "RNA", cellType = "epithelial")
# 
# epi_anndatafile <- file.path(tempdir(), "epithelial.h5ad")
# 
# writeH5AD(sce = epithelial, file = epi_anndatafile)

## ----launchisee, eval=FALSE---------------------------------------------------
# if (require(iSEE)) {
#   iSEE(epithelial)
# }

## -----------------------------------------------------------------------------
sessionInfo()

library("googlesheets4")

gsheet_url <- "https://docs.google.com/spreadsheets/d/1myzlfmyh8QjSha3-c3Z32QtoKUhffdjx0A3rj1wJzac/edit#gid=0"

sloglo_df <- read_sheet(gsheet_url) |>
  as.data.frame()
rownames(sloglo_df) <- sloglo_df$annotation_detailed

sloglo_df

DT::datatable(sloglo_df)

write.table(sloglo_df,
  file = "inst/extdata/sloglo_tabular.csv",
  col.names = TRUE,
  row.names = TRUE
)

# check it reads in correctly
read.table("inst/extdata/sloglo_tabular.csv")

# This script keeps track of all the annotation changes and versions,
# and saves a dictionary to be used in the updateAnnotations function

annotations_dictionary <- list(
  dict_20220215_to_20220619 = c(
    # MBC
    "MBC-like_nonproli" = "Precursor MBCs",
    "MBC-like_proli1" = "Precursor MBCs",
    "MBC-like_proli2"= "Reactivated proliferative MBCs",
    "MBC-like_proli3" = "Reactivated proliferative MBCs",
    "MBC-like_FCRL4+"= "Reactivated proliferative MBCs",
    "PC-precursors" = "PC committed Light Zone GCBC",
    "GC-Tfh-0X40" = "GC-Tfh-OX40",
    "non-GC-Tf-regs" = "Eff-Tregs-IL32",
    "GC-Tf-regs" = "Tfr",
    "IFN CD8 T" = "IFN+ CD8 T",
    
    # GCBC
    "LZ FDC" = "FDC",
    "DZ FDC" = "COL27A1+ FDC",
    "DZ_Sphase" = "DZ early Sphase",
    "DZ_Sphase_HistoneHigh" = "DZ late Sphase",
    "DZ_G2M_HistoneHigh" = "DZ early G2Mphase",
    "DZ_G2M_CCNBHigh"= "DZ late G2Mphase",
    "DZ-cell cycle exit" = "DZ cell cycle exit",
    "DZ-nonproliferative" = "DZ cell cycle exit",
    "DZ-nonproliferative_FOXP1hi"= "DZ non proliferative",
    "DZ/LZ" = "DZ_LZ transition",
    "DZ/LZ" = "DZ_LZ transition",
    "LZ" = "LZ",
    "LZ-BCL2A1 neg"= "LZ",
    "LZ-DZ-re-entry early commitment" = "LZ_DZ reentry commitment",
    "LZ-proliferative_BCL2A1pos" = "LZ proliferative",
    "LZ-proliferative_BCL2A1neg" = "LZ_DZ transition",
    
    # PC
    "class switch MBC" = "csMBC",
    "Neutrophil Granulocytes" = "Neutrophils",
    
    # CD8 T
    "CXCR6+ RM CD8 T" = "RM CD8 activated T",
    
    # myeloid
    "IL7R MMP12 macrophages" = "MMP Slancytes",
    "C1Q HLA macrophages" = "C1Q Slancytes",
    "SELENOP FUCA1 PTGDS macrophages" = "SELENOP Slancytes",
    "ITGAX ZEB2 macrophages" = "ITGAX Slancytes",
    "Mast cells" = "Mast"
  ),
  dict_20220619_to_20230508 = c(
    # Unconventional CD8 T cells
    "MAIT" = "MAIT/Vδ2+ γδ T",
    "TCRVδ+ gd T" = "non-Vδ2+ γδ T",
    "CD56+ gd T" = "ZNF683+ CD8 T",
    "Nksig CD8 T" = "EM CD8 T",
    
    # Slan-like
    "C1Q Slancytes" = "C1Q Slan-like",
    "ITGAX Slancytes" = "ITGAX Slan-like",
    "MMP Slancytes" = "MMP Slan-like",
    "SELENOP Slancytes" = "SELENOP Slan-like",
    
    # NK
    "CD16-CD56- NK" = "CD16-CD56dim NK"
  )
)

save(annotations_dictionary, file = here::here("data/annotations_dictionary.RData"))

# This script creates the colors colors_20230508, so that we can recreate
# UMAPs and violin plots using the same color palettes used in the paper.

colors_20230508 <- list(
  level_1 = c(
    "NBC_MBC" = "#a6cee3",
    "GCBC" = "#398D9F",
    "PC" = "#032872",
    "CD4_T" = "#a50000",
    "Cytotoxic" = "#67253a",
    "myeloid" = "#6ac087",
    "FDC" = "#d5ff00",
    "PDC" = "#f032e6",
    "epithelial" = "#ffdc9f",
    "preBC" = "#45d7c7",
    "preTC" = "#e87281"
  ),
  "NBC-MBC" = c(
    "NBC" = "#dbeef2",
    "NBC early activation" = "#95cce0",
    "NBC IFN-activated" = "#4d828c",
    "NBC CD229+" = "#8a73ae",
    "Early GC-commited NBC" = "#74c9b8",
    "GC-commited NBC" = "#6dcbb4",
    "preGC" = "#6ebf90",
    "Proliferative NBC" = "#205b69",
    "GC DZ Noproli" = "#558e1b",
    "Early MBC" = "#ebe2c3",
    "ncsMBC" = "#dccb9b",
    "ncsMBC FCRL4/5+" = "#bca041",
    "csMBC" = "#6f663a",
    "csMBC FCRL4/5+" = "#a29158",
    "MBC FCRL5+" = "#332d1b"
  ),
  "GCBC" = c(
    "DZ early Sphase"= "#036ebf",
    "DZ late Sphase" = "#004f89",
    "DZ early G2Mphase" = "#6f65a1",
    "DZ late G2Mphase" = "#867fa8",
    "DZ cell cycle exit" = "#2e2d4c",
    "DZ non proliferative" = "#336103",
    "DZ_LZ transition" = "#d0efa1",
    "LZ" = "#fbf397",
    "LZ_DZ reentry commitment" = "#e1efef",
    "LZ proliferative" = "#97d1f8",
    "LZ_DZ transition" = "#4eb0f9",
    "Precursor MBCs" = "#c4b771",
    "Reactivated proliferative MBCs" = "#574b2e",
    "PC committed Light Zone GCBC" = "#fccac3"
  ),
  "PC" = c(
    "Dark Zone GCBC" = "#76797e",             
    "DZ migratory PC precursor" = "#d74403",
    "Light Zone GCBC" = "#b8bac1",
    "PC committed Light Zone GCBC" = "#fccec4",
    "Early PC precursor" = "#fd887d",
    "PB committed early PC precursor" = "#eebecb",
    "Transitional PB" = "#e57d95",
    "PB" = "#c24667",           
    "IgG+ PC precursor" = "#a3e0ee",
    "preMature IgG+ PC" = "#0a8fc8",
    "Mature IgG+ PC" = "#304074",
    "MBC derived IgG+ PC" = "#546a9d",
    "Mature IgA+ PC" = "#4a322e",
    "MBC derived IgA+ PC" = "#866a5a",
    "csMBC" = "#035e6f",
    "MBC derived early PC precursor" = "#418b9d",
    "MBC derived PC precursor" = "#96c2cf",
    "IgM+ early PC precursor" = "#62b160",     
    "IgM+ PC precursor" = "#1a8e18",
    "preMature IgM+ PC" = "#016400",
    "Mature IgM+ PC" = "#013701",
    "Short lived IgM+ PC" = "#92cc92",
    "IgD PC precursor" = "#f3ddc2"
  ),
  "CD4-T" = c(
    "Naive" = "gray88",
    "CM Pre-non-Tfh" = "gray71",
    "CM PreTfh" = "gray59",
    "Tfh T:B border" = "#67a9cf",
    "Tfh-LZ-GC" = "#3690c0",
    "GC-Tfh-SAP" = "#02818a",
    "GC-Tfh-OX40"= "#016c59",
    "Tfh-Mem" = "#014636",
    "T-Trans-Mem" = "#fd8d3c",
    "T-Eff-Mem" = "#e31a1c",
    "T-helper" = "#800026",
    "Eff-Tregs" = "#df65b0",
    "Eff-Tregs-IL32" = "#e7298a",
    "Tfr" = "#ce1256",
    "cycling T" = "#e9001b"
  ),
  "Cytotoxic" = c(
    "Naive CD8 T" = "#ffb6c1",
    "SCM CD8 T" = "#ffc4a6",
    "CM CD8 T" = "#e46c70",
    "RM CD8 T" = "#ce262f",
    "RM CD8 activated T" = "#862222",
    "CD8 Tf" = "#74b2d0",
    "DC recruiters CD8 T" = "#9E8073", 
    "IFN+ CD8 T" = "#441815",
    "EM CD8 T" = "#E58F47",
    "ZNF683+ CD8 T" = "#a2c8c7",
    "non-Vδ2+ γδ T" = "#5ea19e",
    "MAIT/Vδ2+ γδ T" = "#008080",
    "CD16-CD56+ NK" = "#a19d41",
    "CD16-CD5dim- NK" = "#fcf75e",
    "CD16+CD56dim NK" = "#cec94f",
    "ILC1" = "#80eda8",
    "NKp44+ ILC3" = "#6ac18a",
    "NKp44- ILC3" = "#55976d",
    "DN"= "#003b59" 
  ),
  "myeloid" = c(
    "DC1 precursor" = "#ff99d6",
    "DC1 mature" = "#bf408c",
    "DC2" = "#f99fa7",
    "DC3" = "#e06c6c",
    "DC4" = "#8f523d",
    "DC5" = "#d73919",
    "IL7R DC" = "#e1b40e",
    "aDC1" = "#e2f155",
    "aDC2" = "#b4d84b",
    "aDC3" = "#6ead34",
    "M1 Macrophages" = "#3f56ca",
    "Monocytes" = "#cd87de",
    "Mast" = "#825e7b",
    "Neutrophils" = "#aa7c60",
    "Cycling" = "gray",
    "MMP Slan-like" = "#91f8ca",
    "C1Q Slan-like" = "#68cde8",
    "SELENOP Slan-like" = "#519bdb",
    "ITGAX Slan-like" = "#7ef1eb"
  ),
  "FDC" = c(
    "FDC" = "#632c63",
    "cycling FDC" = "#cc66cc",
    "CD14+CD55+ FDC" = "#e4624e",
    "COL27A1+ FDC" = "#92e8df",
    "FRC" = "yellow3",
    "MRC" = "limegreen",
    "unknown" = "#999999"
  ),
  "epithelial" = c(
    "Basal cells" = "#1b9e77",
    "VEGFA+" = "#d95f02",
    "Surface epithelium" = "#7570b3",
    "Outer surface" = "#e7298a",
    "Crypt" = "#66a61e",
    "FDCSP epithelium" = "#e6ab02"
  ),
  "PDC" = c(
    "PDC" = "#f032e6",
    "IFN1+ PDC" = "#510791"
  )
)

save(colors_20230508, file = here::here("data/colors_20230508.RData"))

#' List available cell types for a dataset of the tonsil cell atlas
#'
#' @param assayType Either 'RNA', 'ATAC', 'Multiome', 'CITE', or 'Spatial'
#' @param version Version of the tonsil atlas data: '1.0' (preprint) or '2.0'
#'   or '2.0' (publication, default). Note that, for version 2.0, CD8-T and
#'   ILC-NK are combined in a single 'Cytotoxic' object.
#'
#' @export
#'
#' @return A character vector with the available cell types for the indicated
#'   dataset.
#' @examples
#' listCellTypes(assayType = "RNA", version = "2.0")
#'

listCellTypes <- function(assayType=c("RNA", "ATAC", "Multiome", "CITE", "Spatial"),
                          version="2.0") {
  assayType <- match.arg(assayType)
  version <- match.arg(version, c("1.0", "2.0"))
  if (assayType %in% c("ATAC", "Multiome", "CITE", "Spatial")) {
    cellTypes <- "All"
  } else if(assayType == "RNA") {
    cellTypes <- switch(
      version,
      "1.0" = c("All", "NBC-MBC", "GCBC", "PC","CD4-T", "Th", "CD8-T", "ILC-NK",
                "myeloid", "FDC", "epithelial", "PDC", "preB", "preT"),
      "2.0" = c("All", "NBC-MBC", "GCBC", "PC", "CD4-T", "Cytotoxic", "myeloid",
                "FDC", "epithelial", "PDC")
    )
  }
  cellTypes
}

data("annotations_dictionary")
test_that("downloading RNA data works for both versions", {
  # Version 1
  sce_v1 <- HCATonsilData(
    assayType = "RNA",
    cellType = "epithelial",
    processedCounts = TRUE,
    version = "1.0"
  )
  
  # Version 2
  sce <- HCATonsilData(
    assayType = "RNA",
    cellType = "epithelial",
    processedCounts = TRUE,
    version = "2.0"
  )

  # Test slots and classes
  expect_s4_class(sce_v1, "SingleCellExperiment")
  expect_s4_class(sce, "SingleCellExperiment")
  expect_s4_class(counts(sce), "DelayedMatrix")
  expect_s4_class(SingleCellExperiment::logcounts(sce), "DelayedMatrix")
  expect_equal(length(SingleCellExperiment::reducedDims(sce)), 3)
  expect_equal(names(SingleCellExperiment::reducedDims(sce)), c("PCA", "UMAP", "HARMONY"))
  expect_s4_class(SingleCellExperiment::colData(sce), "DFrame")
  expect_s4_class(SingleCellExperiment::rowData(sce), "DFrame")

  # Test dimensions
  expect_equal(dim(sce), c(37378, 396))
  expect_equal(dim(counts(sce)), c(37378, 396))
  expect_equal(dim(SingleCellExperiment::logcounts(sce)), c(37378, 396))
  expect_equal(dim(SingleCellExperiment::reducedDim(sce, "PCA")), c(396, 50))
  expect_equal(dim(SingleCellExperiment::reducedDim(sce, "UMAP")), c(396, 2))
  expect_equal(dim(SingleCellExperiment::reducedDim(sce, "HARMONY")), c(396, 50))
  expect_equal(dim(SingleCellExperiment::rowData(sce)), c(37378, 3))
  expect_equal(dim(SingleCellExperiment::colData(sce)), c(396, 39))

  # Test variables colData()
  vars <- c("barcode", "donor_id", "gem_id", "library_name", "assay", "sex",
            "age","age_group", "hospital", "cohort_type", "cause_for_tonsillectomy",
            "is_hashed", "preservation", "nCount_RNA", "nFeature_RNA", "pct_mt",
            "pct_ribosomal", "pDNN_hashing", "pDNN_scrublet", "pDNN_union",
            "scrublet_doublet_scores", "S.Score", "G2M.Score", "Phase",
            "scrublet_predicted_doublet", "doublet_score_scDblFinder",
            "annotation_level_1", "annotation_level_1_probability",
            "annotation_figure_1", "annotation_20220215", "annotation_20220619",
            "annotation_20230508", "annotation_20230508_probability",
            "UMAP_1_level_1", "UMAP_2_level_1", "UMAP_1_20220215", "UMAP_2_20220215",
            "UMAP_1_20230508", "UMAP_2_20230508")
  expect_true(all(vars %in% colnames(SingleCellExperiment::colData(sce))))

  # Test that HCATonsilData generates errors if inputs are incorrect
  expect_error(HCATonsilData(assayType = "DNA"), regexp = NULL)
  expect_error(
    HCATonsilData(assayType = "RNA", cellType = "neuron"),
    regexp = NULL
  )

  # Test that logcounts are not included if processedCounts = FALSE
  sce2 <- HCATonsilData(
    assayType = "RNA",
    cellType = "epithelial",
    processedCounts = FALSE
  )
  expect_error(SingleCellExperiment::logcounts(sce2), regexp = NULL)

  # Test that

})


test_that("All cellTypes exist in ExperimentHub", {
  eh <- ExperimentHub::ExperimentHub()
  cellTypes <- HCATonsilData::listCellTypes("RNA", version = "2.0")
  out <- all(vapply(cellTypes, \(.) any(grepl(., eh$rdatapath)), logical(1)))
  expect_true(out)
  cellTypes <- HCATonsilData::listCellTypes("RNA", version = "1.0")
  out <- all(vapply(cellTypes, \(.) any(grepl(., eh$rdatapath)), logical(1)))
  expect_true(out)
})

# test_that("Info retrieval", {
#   expect_message(
#     info_sce <- HCATonsilDataInfo(assayType = "RNA", cellType = "epithelial")
#   )
#   expect_type(info_sce, "character")
#   expect_true(grepl(pattern = "277 cells", info_sce))
# })


test_that("Updating annotations works", {
  sce <- HCATonsilData(
    assayType = "RNA",
    cellType = "myeloid",
    processedCounts = FALSE,
    version = "1.0"
  )
  # 20220215 --> 20220619
  sce <- updateAnnotation(
    sce = sce,
    refAnnotation = "20220215",
    newAnnotation = "20220619"
  )
  oldAnnot <- sce$annotation_20220215
  newAnnot <- sce$annotation_20220619
  expect_true(
    all(colnames(sce)[oldAnnot == "SELENOP FUCA1 PTGDS macrophages"] ==
        colnames(sce)[newAnnot == "SELENOP Slancytes"])
  )
  
  # 20220619 --> 20230508
  sce <- updateAnnotation(
    sce = sce,
    refAnnotation = "20220619",
    newAnnotation = "20230508"
  )
  oldAnnot <- sce$annotation_20220619
  newAnnot <- sce$annotation_20230508
  expect_true(
    all(colnames(sce)[oldAnnot == "SELENOP FUCA1 PTGDS macrophages"] ==
          colnames(sce)[newAnnot == "SELENOP Slan-like"])
  )
})

#' TonsilData_glossary
#'
#' Convenience function to read directly in the file provided as `extdata`
#'
#' @return Data frame containing the info on the cell types included in the
#' TonsilDataAtlas
#' @export
#'
#' @examples
#' glossary_df <- TonsilData_glossary()
#' head(glossary_df)
TonsilData_glossary <- function() {
  glossary_location <- system.file("extdata", "sloglo_tabular.csv", package = "HCATonsilData")

  glossary_df <- read.table(glossary_location)

  return(glossary_df)
}




#' TonsilData_cellinfo
#' 
#' Query the glossary to traceback the rationale for each annotation of the
#' tonsil atlas
#'
#' @param cellType String character, used to define the cell type for which the
#' information will be displayed. Defaults to `NULL` - if left unspecified, the
#' function returns a list of the possible options
#'
#' @return Invisible `NULL` - the information is displayed as a message in the
#' console.
#'
#' @export
#'
#' @importFrom utils browseURL read.table
#'
#' @examples
#' TonsilData_cellinfo()
#'
#' TonsilData_cellinfo("PDC")
TonsilData_cellinfo <- function(cellType = NULL) {

  glossary_df <- TonsilData_glossary()
  slo_celltypes <- rownames(glossary_df)

  if (is.null(cellType)) {
    message("Please select one of the following: ",
            paste0(slo_celltypes, collapse = "|"))
  } else {

    if (cellType %in% slo_celltypes) {

      cell_msg <- paste0(
        glossary_df[cellType, "annotation_detailed"],
        "\n------------------------------",
        "\nAnnotation Level 1: ",
        glossary_df[cellType, "annotation_level_1"],
        "\nCell Markers: ",
        glossary_df[cellType, "description"],
        "\nCell Markers: ",
        glossary_df[cellType, "marker_genes"],
        "\nRelated references: ",
        glossary_df[cellType, "related_refs"] # ,
        # "\nCell Ontology terms: ",
        # glossary_df[matched_celltype, "related_cellontology"]
      )
      message(cell_msg)

    } else {
      message("Cell type not found! Please select one of the following: ",
              paste0(slo_celltypes, collapse = "|"))
    }
  }

  invisible(NULL)
}


.actionbutton_biocstyle <- "color: #ffffff; background-color: #0092AC"

.link_marker <- function(gene_id) {
  sprintf(
    '<a href = "http://www.ncbi.nlm.nih.gov/gene/?term=%s[sym]" target = "_blank" class = "btn btn-primary" style = "%s">%s</a>',
    gene_id,
    .actionbutton_biocstyle,
    gene_id
  )
}

.link_cellontology <- function(cl_id) {
  # https://www.ebi.ac.uk/ols/ontologies/cl/terms?iri=http%3A%2F%2Fpurl.obolibrary.org%2Fobo%2FCL_0000084
  # actually this one is better: http://purl.obolibrary.org/obo/CL_0000000 (and works)

  cl_term <- gsub("CL:", "CL_", cl_id)

  sprintf(
    '<a href = "http://purl.obolibrary.org/obo/%s" target = "_blank" class = "btn btn-primary" style = "%s">%s</a>',
    cl_term,
    .actionbutton_biocstyle,
    cl_id
  )
}

.link_refs <- function(refname, ref_doi) {
  sprintf(
    '<a href = "https://doi.org/%s" target = "_blank" class = "btn btn-primary" style = "%s">%s</a>',
    ref_doi,
    .actionbutton_biocstyle,
    refname
  )
}

#' TonsilData_cellinfo_html
#'
#' @param cellType String character, used to define the cell type for which the
#' information will be displayed. Defaults to `NULL` - if left unspecified, the
#' function returns a list of the possible options
#' @param display_plot Logical value, defines whether to include or not a plot
#' for the UMAP of the selected cell type
#' @param output_to Character value, one of "single_page" or "html_to_embed".
#' Defines in which form the information is returned, either as an individual page
#' or simply as HTML code to directly embed into other documents.
#'
#' @return Either the HTML code generated to be embedded, or a standalone HTML
#' page is created - and the location to this temp file is returned as a character
#' (default).
#' @export
#'
#' @importFrom htmltools tags
#' @importFrom utils browseURL read.table
#' @importFrom rmarkdown render
#' @importFrom base64enc dataURI
#'
#' @examples
#' TonsilData_cellinfo_html("PDC")
#' TonsilData_cellinfo_html("preB")
#' TonsilData_cellinfo_html("preT")
#' TonsilData_cellinfo_html("preT", output_to = "html_to_embed")
TonsilData_cellinfo_html <- function(cellType = NULL,
                                     display_plot = TRUE,
                                     output_to = c("single_page", "html_to_embed")) {

  output_to <- match.arg(output_to, c("single_page", "html_to_embed"))

  glossary_df <- TonsilData_glossary()
  slo_celltypes <- rownames(glossary_df)

  if (is.null(cellType)) {
    message("Please select one of the following: ",
            paste0(slo_celltypes, collapse = "|"))
  } else {

    if (cellType %in% slo_celltypes) {

      cell_html_celltype <- paste0(
        tags$b(cellType),
        tags$br(),
        tags$b("Annotation Level 1: "),
        glossary_df[cellType, "annotation_level_1"],
        tags$hr(),
        tags$br()
      )

      cell_html_celldescription <- paste0(
        tags$b("Cell type description: "),
        glossary_df[cellType, "description"],
        tags$br(), tags$br()
      )

      markers <- as.character(glossary_df[cellType, "marker_genes"])
      markers_split <- unlist(strsplit(markers, split = ",", fixed = TRUE))
      markers_content <- paste(
        unlist(lapply(markers_split, function(id) .link_marker(id))),
        collapse = " ")

      cell_html_cellmarkers <- paste0(
        tags$b("Marker genes for this cell type: "),
        tags$br(),
        markers_content,
        tags$br(),
        tags$br()
      )

      refs <- as.character(glossary_df[cellType, "related_refs"])
      refs_split <- unlist(strsplit(refs, split = ";", fixed = TRUE))
      refs_split_names <- unlist(lapply(
        strsplit(refs_split, split = "|", fixed = TRUE),
        function(arg) arg[1]))
      refs_split_dois <- unlist(lapply(
        strsplit(refs_split, split = "|", fixed = TRUE),
        function(arg) arg[2]))

      refs_content <- paste(
        unlist(lapply(seq_len(length(refs_split)), function(i) {
          refname <- refs_split_names[i]
          ref_doi <- refs_split_dois[i]
          .link_refs(refname = refname, ref_doi = ref_doi)
        })),
        collapse = " ")

      cell_html_refs <- paste0(
        tags$b("References: "),
        refs_content,
        tags$br(),
        tags$br()
      )

      # cellonts <- as.character(glossary_df[cellType, "related_cellontology"])
      # cellonts_split <- unlist(strsplit(cellonts, split = ",", fixed = TRUE))
      # cellonts_content <- paste(
      #   unlist(lapply(cellonts_split, function(id) .link_cellontology(id))),
      #   collapse = " ")
      #
      # cell_html_cellontology <- paste0(
      #   tags$b("Cell ontology terms: "),
      #   cellonts_content,
      #   tags$br()
      # )

      # UMAP plot
      if (display_plot) {
        img_location <- glossary_df[cellType, "umap_png"]
        img_html <-
          paste0(
            tags$img(
              src = base64enc::dataURI(
                file = system.file("images", img_location, package = "HCATonsilData"),
                mime = "image/png"
              ),
              width = 500
            ),
            tags$hr()
          )
      } else {
        img_html <- ""
      }

      cell_html <- paste0(
        cell_html_celltype,
        img_html,
        cell_html_celldescription,
        cell_html_cellmarkers,
        cell_html_refs #,
        # cell_html_cellontology
      )

      if (output_to == "single_page") {
        # put this into a html file
        tmpfile <- tempfile()

        writeLines(cell_html, con = tmpfile)

        # and render this
        browseURL(rmarkdown::render(tmpfile))
        return(tmpfile)
      } else if (output_to == "html_to_embed") {
        return(cell_html)
      }

    } else {
      message("Cell type not found! Please select one of the following: ",
              paste0(slo_celltypes, collapse = "|"))
    }
  }
}

test_that("loading glossary", {
  glossary_df <- TonsilData_glossary()
  expect_s3_class(glossary_df, "data.frame")

  expect_true(ncol(glossary_df) == 7)
})


test_that("Picking up info on cell type", {

  expect_message(TonsilData_cellinfo("preB"))
  expect_message(TonsilData_cellinfo())
  expect_message(TonsilData_cellinfo("Some cell not in there"))

  expect_true(
    is(TonsilData_cellinfo_html("Mature IgG+ PC"), "character")
  )

  expect_true(
    is(TonsilData_cellinfo_html("Mature IgG+ PC", output_to = "html_to_embed"), "character")
  )

  expect_message(
    TonsilData_cellinfo_html()
  )

  expect_message(
    TonsilData_cellinfo_html("A weird cell type")
  )


})

#' Update cell type annotation
#'
#' Annotations are dynamic by nature. As more experts look into the data and
#' newer literature comes out, we expect annotations to be refined over time.
#' We have accommodated this by allowing us and users to add new annotations
#' to the SingleCellExperiment objects. If you want to propose a new
#' annotation based on your experience or new evidence, please open an issue
#' at https://github.com/massonix/HCATonsilData/issues.
#'
#' @param sce A SingleCellExperiment object obtained using HCATonsilData
#'   function.
#' @param refAnnotation string specifying the date of the annotation to use as
#'   reference.
#' @param newAnnotation string specifying the suffix to add to the new column
#'   (annotation_*).
#'
#' @return A SingleCellExperiment object with additional an additional column
#'   (annotation_*) that contains more annotations.
#' @export
#'
#' @examples
#' # update the annotation from preprint (version 1.0) to publication (version 2.0)
#' \dontrun{
#' sce <- updateAnnotation(
#'   sce,
#'   refAnnotation = "20220619",
#'   newAnnotation = "20230508"
#' )
#' }
updateAnnotation <- function(sce,
                             refAnnotation = "20220215",
                             newAnnotation = "20220619") {
  oldAnnot <- paste("annotation", refAnnotation, sep = "_")
  newAnnot <- paste("annotation", newAnnotation, sep = "_")
  selectedDict <- paste0("dict_", refAnnotation, "_to_", newAnnotation)
  selectedDict <- annotations_dictionary[[selectedDict]]
  sce[[newAnnot]] <- as.character(sce[[oldAnnot]])
  selectedDict <- selectedDict[names(selectedDict) %in% unique(sce[[oldAnnot]])]
  for (cellType in names(selectedDict)) {
    sce[[newAnnot]][sce[[newAnnot]] == cellType] <- selectedDict[cellType]
  }
  sce
}

globalVariables("annotations_dictionary")

# This script generates the metadata.csv file for the Spatial data as requested in the following
# tutorial: https://bioconductor.org/packages/devel/bioc/vignettes/HubPub/inst/doc/CreateAHubPackage.html
# Note that this script is inspired in https://github.com/fmicompbio/TabulaMurisSenisData/blob/master/inst/scripts/make-metadata-tabula-muris-senis-droplet.R

# Load packages
library(dplyr)
library(tidyr)
library(readr)
library(stringr)
library(here)
library(glue)
library(purrr)


# Get data slots
data_dir <- here("inst/scripts/HCATonsilData/2.0/Spatial")
slots <- list.files(data_dir)


# Generate metadata file for ExperimentHub package
slots_no_image <- slots %>%
  str_subset("(image|scale)", negate = TRUE)
slots_names <- slots_no_image %>%
  str_split("_") %>%
  map_chr(\(x) str_subset(x, "(.rds|.h5)")) %>%
  str_remove("(.rds|.h5)")
titles <- c(counts = "counts", logcounts = "processed counts", coldata = "colData",
            rowdata = "rowData", pca = "PCA", harmony = "HARMONY", umap = "UMAP")
descs <- c(counts = "Count matrix", logcounts = "Processed count matrix", coldata = "Cell metadata",
           rowdata = "Gene annotation", pca = "PCA representation", harmony = "HARMONY representation",
           umap = "UMAP representation")
rdclass <- c(counts = "H5File", logcounts = "H5File", coldata = "DFrame",
             rowdata = "DFrame", pca = "matrix", harmony = "matrix",
             umap = "matrix")
names(slots_no_image) <- slots_names
metadata_df1 <- data.frame(
  Title = glue("Tonsil Atlas Visium data {titles[slots_names]}"),
  Description = glue("{descs[slots_names]} for the Tonsil Cell Atlas Visium dataset"),
  RDataPath = glue("HCATonsilData/2.0/Spatial/{slots_no_image[slots_names]}"),
  RDataClass = rdclass[slots_names],
  DispatchClass = ifelse(rdclass[slots_names] == "H5File", "H5File", "Rds")
)


# Handle images and scales
slots_image <- str_subset(slots, "(image|scale)")
sample_ids <- slots_image %>%
  str_remove("(Spatial_image_|Spatial_scale_)") %>%
  str_remove(".rds")
file_type <- str_extract(slots_image, "(scale|image)")
file_type[file_type == "image"] <- "H&E staining"
metadata_df2 <- data.frame(
  Title = glue("Tonsil Atlas Visium {file_type} ({sample_ids})"),
  Description = glue("{file_type} for sample {sample_ids}"),
  RDataPath = glue("HCATonsilData/2.0/Spatial/{slots_image}"),
  RDataClass = ifelse(file_type == "scale", "scalefactors", "raster"),
  DispatchClass = "Rds"
)
metadata_df <- bind_rows(metadata_df1, metadata_df2)
metadata_df <- metadata_df %>%
  mutate(
    BiocVersion = "3.18",
    Genome = "GRCh38",
    SourceType = "HDF5",
    SourceUrl = "https://zenodo.org/record/8373756",
    SourceVersion = "2.0",
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = NA,
    DataProvider = "BCLL@las",
    Maintainer = "Ramon Massoni-Badosa <ramonmassoni@gmail.com>"
) %>%
  select(Title, Description, RDataPath, BiocVersion, Genome,
         SourceType, SourceUrl, SourceVersion, Species, TaxonomyId,
         Coordinate_1_based, DataProvider, Maintainer, RDataClass,
         DispatchClass)
rownames(metadata_df) <- NULL


# Write
write_delim(
  metadata_df,
  file = here("inst/extdata/metadata-tonsil-atlas-spatial-v2.csv"),
  delim = ","
)

#' Annotation dictionary for naive and memory B cells (NBC-MBC)
#'
#' In version 1 of the atlas, we changed the clusters and annotation for NBC-MBC
#' prior to publishing the preprint. Since, there is not a 1:1 mapping between
#' clusters, here we provide the correspondence between the annotations in
#' February 2022 and the preprint (June 2022) for each NBC-MBC cell barcode.
#'
#' This data is used by the updateAnnotation function in B cells.
#'
#'
#' @format A dataframe with 112478 NBC-MBC and 5 variables
#' \describe{
#'   \item{barcode}{cell barcode}
#'   \item{names_level_5_clusters_eta}{Cell annotation given by the B-cell annotation team in February 2022}
#'   \item{annotation_20220619}{Cell annotation given by the B-cell annotation team June 2022 (preprint)}
#'   \item{UMAP_1}{UMAP1 coordinates}
#'   \item{UMAP_2}{UMAP2 coordinates}
#' }
#'
#' @source \url{https://zenodo.org/record/8373756}
#' @name NBC_MBC_annotation_df
#' @docType data
globalVariables("NBC_MBC_annotation_df")



#' Annotation dictionary to keep track of annotations changes and versions
#'
#' Annotations are opinionated and dynamic by nature. They are subjected to
#' change as more experts look into the data and more evidence is published.
#' HCATonsilData has been designed to account for that. We timestamp annotations
#' and plan to include additional annotations if users propose them and we
#' validate them.
#'
#'
#' @format A list of named vectors with the correspondence between time-stamped
#'   annotations
#'
#' @source This file was created with the script in
#' "inst/scripts/make-annotation-dictionary.R
#' @name annotations_dictionary
#' @docType data
globalVariables("annotations_dictionary")

#' Donor metadata
#'
#' Data frame that contains all metadata information for the 17 donors included
#' in the tonsil atlas.
#'
#'
#' @format A dataframe with 17 observations and 8 variables
#'
#' @source Check the tonsil atlas publication
#' @name donor_metadata
#' @docType data
globalVariables("donor_metadata")


#' Color palettes
#'
#' Color palettes used in the paper for all cell types
#'
#'
#' @format A list with the HEX color codes for each cell type
#'
#' @source This file was created with the script in
#' "inst/scripts/make-annotation-colors.R
#' @name colors_20230508
#' @docType data
globalVariables("colors_20230508")

#' Access the Tonsil Atlas data (RNA, ATAC, Multiome, CITE, Spatial)
#'
#' The data was downloaded from Zenodo
#' https://zenodo.org/record/8373756
#'
#' @param assayType One of 'RNA', 'ATAC', 'Multiome', 'CITE' or 'Spatial'.
#' @param cellType A character vector of length 1 with the desired cell type.
#'    A list of available cell types can be obtained using
#'    \code{listCellTypes(assay_type)}.
#' @param processedCounts Logical scalar. If \code{TRUE}, include the processed
#'   (normalized) counts in addition to the raw counts in the
#'   SingleCellExperiment object.
#' @param version Version of the tonsil atlas data to retrieve: "1.0" (preprint)
#'   or "2.0" (publication, default)
#'
#' @return A \linkS4class{SingleCellExperiment} object for the cellType
#'   requested. For scATAC-seq, Multiome and CITE we provide the instructions
#'   for downloading the Seurat objects in Zenodo (see vignette)
#'
#' @author Ramon Massoni-Badosa

#'
#' @importFrom ExperimentHub ExperimentHub
#' @importFrom HDF5Array HDF5Array
#' @importFrom S4Vectors DataFrame
#' @importFrom SingleCellExperiment SingleCellExperiment reducedDim reducedDim<-
#'   reducedDims
#' @importFrom SpatialExperiment SpatialExperiment SpatialImage
#' @importFrom SummarizedExperiment assay
#' @export
#'
#' @examples
#' # retrieve the epithelial scRNA-seq dataset
#' sce_epithelial <- HCATonsilData(
#'   assayType = "RNA",
#'   cellType = "epithelial"
#' )
#' sce_epithelial

HCATonsilData <- function(assayType = c("RNA", "ATAC", "CITE", "Spatial"),
                          cellType = "All",
                          version = "2.0",
                          processedCounts = TRUE) {

  # Check validity of input arguments
  assayType <- match.arg(assayType)
  version <- match.arg(version, c("1.0", "2.0"))
  cellTypes <- listCellTypes(assayType, version)
  cellType <- match.arg(cellType, cellTypes)

  # Point users to the vignette to download scATAC-seq, Multiome or CITE-seq data
  if (assayType %in% c("ATAC", "Multiome", "CITE")) {
    message(
      sprintf(
        "Check the corresponding section in the vignette to download %s data",
        assayType
      )
    )
    message("Please run browseVignettes('HCATonsilData')")
  }

  # Download scRNA-seq data and return a SingleCellExperiment if assay is RNA
  if (assayType == "RNA") {
    # Initialize ExperimentHub object
    eh <- ExperimentHub::ExperimentHub()
    host <- file.path("HCATonsilData", version, assayType)

    # Define paths to files in ExperimentHub
    suffixes <- c("counts.h5", "processed.h5", "rowdata.rds", "coldata.rds",
                  "pca.rds", "harmony.rds", "umap.rds")
    names(suffixes) <- c("counts", "processedCounts", "rowData", "colData",
                         "pca", "harmony", "umap")
    filePaths <- file.path(host, paste(cellType, assayType, suffixes, sep = "_"))
    names(filePaths) <- names(suffixes)
    for (x in filePaths) {
      if (sum(x == eh$rdatapath) > 1) {
        stop(x, " matched more than one entry!")
      }
      if (sum(x == eh$rdatapath) == 0) {
        stop(x, " is not present in ExperimentHub!")
      }
    }

    # Create SingleCellExperiment
    cts <- HDF5Array::HDF5Array(
      eh[eh$rdatapath == filePaths["counts"]][[1]],
      name = "counts"
    )
    sce <- SingleCellExperiment::SingleCellExperiment(
      assays = list(counts = cts),
      rowData = eh[eh$rdatapath == filePaths["rowData"]][[1]],
      colData = eh[eh$rdatapath == filePaths["colData"]][[1]]
    )
    SingleCellExperiment::reducedDims(sce) <- list(
      PCA = eh[eh$rdatapath == filePaths["pca"]][[1]],
      UMAP = eh[eh$rdatapath == filePaths["umap"]][[1]]
    )
    if (sum(filePaths["harmony"] == eh$rdatapath) == 1) {
      harmony <- eh[eh$rdatapath == filePaths["harmony"]][[1]]
      SingleCellExperiment::reducedDims(sce)[["HARMONY"]] <- harmony
    }
    if (processedCounts) {
      prccts <- HDF5Array::HDF5Array(
        eh[eh$rdatapath == filePaths["processedCounts"]][[1]],
        name = "processed"
      )
      SummarizedExperiment::assay(sce, "logcounts", withDimnames = FALSE) <- prccts
    }

    if (version == "1.0") {
      # Update annotations
      sce <- updateAnnotation(sce = sce, "20220215", "20220619")
      sce <- updateAnnotation(sce = sce, "20220619", "20230508")

      # For NBC-MBC, we reclustered them from 20220215 to 20220619, which means
      # that there is not a 1:1 mapping between annotations. Let us reannotate
      # using a pre-loaded dataframe. Also, let's change the UMAP coords
      # to match the paper
      if (cellType == "NBC-MBC") {
        sce <- sce[, NBC_MBC_annotation_df$barcode]
        sce$annotation_20220619 <- NBC_MBC_annotation_df$annotation_20220619
        umap_df <- as.matrix(
          NBC_MBC_annotation_df[, c("UMAP_1", "UMAP_2")]
        )
        reducedDim(sce, "UMAP") <- umap_df
      }

      # Similarly, let us update NBC/MBC annotation in case cellType = All
      if (cellType == "All") {
        annot <- sce$annotation_20220619
        names(annot) <- colnames(sce)
        annot[NBC_MBC_annotation_df$barcode] <- NBC_MBC_annotation_df$annotation_20220619
        sce$annotation_20220619 <- annot
      }
    }
  }

  # Get Visium data and return SpatialExperiment if assaytype is Spatial
  if (assayType == "Spatial") {
    if (version == "1.0") {
      stop("Spatial transcriptomics data is only available for version 2.0")
    }

    # Initialize ExperimentHub object
    eh <- ExperimentHub::ExperimentHub()
    host <- file.path("HCATonsilData", version, assayType)

    # Get raw and processed counts
    cts <- HDF5Array::HDF5Array(
      eh[eh$rdatapath == file.path(host, "Spatial_assay_counts.h5")][[1]],
      name = "counts"
    )
    as <- list(counts = cts)
    if (processedCounts) {
      prccts <- HDF5Array::HDF5Array(
        eh[eh$rdatapath == file.path(host, "Spatial_assay_logcounts.h5")][[1]],
        name = "logcounts"
      )
      as[["logcounts"]] <- prccts
    }

    # Get dimensionality reductions
    dr_filt <- grepl(file.path(host, "Spatial_dimred"), eh$rdatapath)
    dr_paths <- eh$rdatapath[dr_filt]
    names(dr_paths) <- sub(".*/Spatial_dimred_(\\w+)\\.rds", "\\1", dr_paths)
    dr <- lapply(dr_paths, \(x) eh[eh$rdatapath == x][[1]])
    names(dr) <- toupper(names(dr))

    # Get cell and gene metadata
    rowdata <- eh[eh$rdatapath == file.path(host, "Spatial_rowdata.rds")][[1]]
    coldata <- eh[eh$rdatapath == file.path(host, "Spatial_coldata.rds")][[1]]

    # Get images
    img_filt <- grepl(file.path(host, "Spatial_image"), eh$rdatapath)
    sfs_filt <- grepl(file.path(host, "Spatial_scale"), eh$rdatapath)
    img_paths <- eh$rdatapath[img_filt]
    sfs_paths <- eh$rdatapath[sfs_filt]
    names(img_paths) <- sub(".*/Spatial_image_(\\w+)\\.rds", "\\1", img_paths)
    names(sfs_paths) <- sub(".*/Spatial_scale_(\\w+)\\.rds", "\\1", sfs_paths)
    sfs_paths <- sfs_paths[names(img_paths)] # ensure images and scale are ordered equally
    images <- mapply(\(img_path, sfs_path, nm) {
      img <- eh[eh$rdatapath == img_path][[1]]
      sfs <- eh[eh$rdatapath == sfs_path][[1]]
      df <- DataFrame(
        sample_id = nm,
        image_id = "lowres",
        data = I(list(SpatialImage(img))),
        scaleFactor = sfs$lowres
      )
      df
    }, img_path=img_paths, sfs_path = sfs_paths, nm=names(img_paths), SIMPLIFY = TRUE)
    images <- do.call(rbind, images)

    # Create SpatialExperiment object
    sce <- SpatialExperiment(
      assays = as,
      mainExpName = "Spatial",
      reducedDims = dr,
      imgData = images,
      rowData = rowdata,
      colData = coldata,
      spatialCoordsNames = c("imagecol", "imagerow")
    )
  }

  # Return
  return(sce)
}


#' #' Information on the Tonsil Data
#' #'
#' #' Get information on the individual datasets of the Tonsil Atlas data
#' #'
#' #' @inheritParams HCATonsilData
#' #'
#' #' @return An info message is generated, and its content is returned invisibly.
#' #'
#' #' @author Federico Marini
#'
#' #' @export
#' #'
#' #' @examples
#' #' HCATonsilDataInfo(assayType = "RNA", cellType = "epithelial")
#' HCATonsilDataInfo <- function(assayType = c("RNA", "ATAC", "CITE", "Spatial"),
#'                               cellType = listCellTypes(assayType = assayType)) {
#'
#'   # Check validity of input arguments
#'   assayType <- match.arg(assayType)
#'   cellType <- match.arg(cellType)
#'
#'   # Initialize ExperimentHub object
#'   eh <- ExperimentHub::ExperimentHub()
#'   host <- file.path("HCATonsilData", "1.0", assayType)
#'
#'   # Define paths
#'   suffixes <- c("counts.h5", "processed.h5", "rowdata.rds", "coldata.rds",
#'                 "pca.rds", "harmony.rds", "umap.rds")
#'   names(suffixes) <- c("counts", "processedCounts", "rowData", "colData",
#'                        "pca", "harmony", "umap")
#'   filePaths <- vapply(suffixes, \(.) {
#'     x <- file.path(host, paste(cellType, assayType, ., sep = "_"))
#'     x
#'   }, character(1))
#'   for (x in filePaths) {
#'     if (sum(x == eh$rdatapath) > 1) {
#'       stop("Input matched more than one entry!")
#'     }
#'   }
#'
#'   # Fetch info instead of creating full sce object
#'   rd_sce <- eh[eh$rdatapath == filePaths["rowData"]][[1]]
#'   cd_sce <- eh[eh$rdatapath == filePaths["colData"]][[1]]
#'
#'   # buildup the information string
#'   info_sce <- sprintf(
#'     "Dataset: %s (assay selected: %s)\nMeasurements available for %d genes in %d cells",
#'     cellType,
#'     assayType,
#'     nrow(rd_sce),
#'     nrow(cd_sce)
#'   )
#'
#'   message(info_sce)
#'
#'   return(invisible(info_sce))
#' }

# This script generates the metadata_rna.csv file as requested in the following
# tutorial: https://bioconductor.org/packages/devel/bioc/vignettes/HubPub/inst/doc/CreateAHubPackage.html
# Note that this script is inspired in https://github.com/fmicompbio/TabulaMurisSenisData/blob/master/inst/scripts/make-metadata-tabula-muris-senis-droplet.R

# Load packages
library(dplyr)
library(tidyr)
library(readr)
library(stringr)
library(here)
library(glue)


# Read data
path_to_csv <- here("inst/scripts/tonsil-atlas-rds-files.csv")
path_to_data <- here("inst/scripts/HCATonsilData")
files_df <- read.csv(file = path_to_csv, header = TRUE)
files_df <- files_df[files_df$dataset == "RNA", ]


# Generate metadata file for ExperimentHub package
convs <- c(coldata = "colData", counts = "counts", processed = "processed counts",
           rowdata = "rowData", pca = "PCA", harmony = "HARMONY", umap = "UMAP")
convs2 <- c(coldata = "Cell metadata", counts = "Count matrix",
            processed = "Processed count matrix", rowdata = "Gene annotation",
            pca = "PCA representation", harmony = "HARMONY representation",
            umap = "UMAP representation")
suffix <- c(coldata = ".rds", counts = ".h5", processed = ".h5", rowdata = ".rds",
            pca = ".rds", harmony = ".rds", umap = ".rds")
rdclass <- c(coldata = "DFrame", counts = "H5File", processed = "H5File",
             rowdata = "DFrame", pca = "matrix", harmony = "matrix", umap = "matrix")
out_df <- files_df %>%
  mutate(outs = "coldata;counts;processed;rowdata;pca;harmony;umap") %>%
  separate_rows(outs, sep = ";") %>%
  mutate(
    outs2 = convs[outs],
    descs = convs2[outs],
    suffix = suffix[outs]
  ) %>%
  mutate(cell_type = str_replace(cell_type, "(/| )", "_")) %>%
  mutate(
    Title = glue("Tonsil Atlas RNA {cell_type} {outs2}"),
    Description = glue("{descs} for the Tonsil Cell Atlas {cell_type} scRNA-seq/Multiome dataset"),
    RDataPath = glue("HCATonsilData/1.0/RNA/{cell_type}_{dataset}_{outs}{suffix}"),
    BiocVersion = "3.15",
    Genome = "GRCh38",
    SourceType = "HDF5",
    SourceUrl = "https://zenodo.org/record/6340174",
    SourceVersion = "1.0",
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = NA,
    DataProvider = "BCLL@las",
    Maintainer = "Ramon Massoni-Badosa <ramonmassoni@gmail.com>",
    RDataClass = rdclass[outs],
    DispatchClass = ifelse(suffix == ".h5", "H5File", "Rds")
  ) %>%
  select(Title, Description, RDataPath, BiocVersion, Genome,
         SourceType, SourceUrl, SourceVersion, Species, TaxonomyId,
         Coordinate_1_based, DataProvider, Maintainer, RDataClass,
         DispatchClass)
dir.create(here("inst/extdata"))


# Check that all files exist
files <- file.path("HCATonsilData", list.files(path_to_data, recursive = TRUE))
out_df <- out_df[out_df$RDataPath %in% files, ]


# Write
write_delim(
  out_df,
  file = here("inst/extdata/metadata-tonsil-atlas-rna.csv"),
  delim = ","
)

# This script generates the metadata_rna.csv file as requested in the following
# tutorial: https://bioconductor.org/packages/devel/bioc/vignettes/HubPub/inst/doc/CreateAHubPackage.html
# Note that this script is inspired in https://github.com/fmicompbio/TabulaMurisSenisData/blob/master/inst/scripts/make-metadata-tabula-muris-senis-droplet.R

# Load packages
library(dplyr)
library(tidyr)
library(readr)
library(stringr)
library(here)
library(glue)


# Read data
data_dir <- here("inst/scripts/HCATonsilData/2.0/RNA")
slots <- list.files(data_dir)
cell_types <- slots %>%
  str_split("_") %>%
  map_chr(1) %>%
  unique()
df <- data.frame(cell_type = cell_types)


# Generate metadata file for ExperimentHub package
convs <- c(coldata = "colData", counts = "counts", processed = "processed counts",
           rowdata = "rowData", pca = "PCA", harmony = "HARMONY", umap = "UMAP")
convs2 <- c(coldata = "Cell metadata", counts = "Count matrix",
            processed = "Processed count matrix", rowdata = "Gene annotation",
            pca = "PCA representation", harmony = "HARMONY representation",
            umap = "UMAP representation")
suffix <- c(coldata = ".rds", counts = ".h5", processed = ".h5", rowdata = ".rds",
            pca = ".rds", harmony = ".rds", umap = ".rds")
rdclass <- c(coldata = "DFrame", counts = "H5File", processed = "H5File",
             rowdata = "DFrame", pca = "matrix", harmony = "matrix", umap = "matrix")
out_df <- df %>%
  mutate(outs = "coldata;counts;processed;rowdata;pca;harmony;umap") %>%
  separate_rows(outs, sep = ";") %>%
  mutate(
    outs2 = convs[outs],
    descs = convs2[outs],
    suffix = suffix[outs]
  ) %>%
  mutate(cell_type = str_replace(cell_type, "(/| )", "_")) %>%
  mutate(
    Title = glue("Tonsil Atlas RNA {cell_type} {outs2}"),
    Description = glue("{descs} for the Tonsil Cell Atlas {cell_type} scRNA-seq/Multiome dataset"),
    RDataPath = glue("HCATonsilData/2.0/RNA/{cell_type}_{dataset}_{outs}{suffix}"),
    BiocVersion = "3.18",
    Genome = "GRCh38",
    SourceType = "HDF5",
    SourceUrl = "https://zenodo.org/record/8373756",
    SourceVersion = "2.0",
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = NA,
    DataProvider = "BCLL@las",
    Maintainer = "Ramon Massoni-Badosa <ramonmassoni@gmail.com>",
    RDataClass = rdclass[outs],
    DispatchClass = ifelse(suffix == ".h5", "H5File", "Rds")
  ) %>%
  select(Title, Description, RDataPath, BiocVersion, Genome,
         SourceType, SourceUrl, SourceVersion, Species, TaxonomyId,
         Coordinate_1_based, DataProvider, Maintainer, RDataClass,
         DispatchClass)

# Check that all files exist
if (all(str_remove(out_df$RDataPath, "HCATonsilData/2.0/RNA/") %in% slots)) {
  print("All files exist and are ready to be uploaded")
} else {
  print("Missing files!")
}


# Write
write_delim(
  out_df,
  file = here("inst/extdata/metadata-tonsil-atlas-rna-v2.csv"),
  delim = ","
)

---
title: "Download and preprocess the tonsil atlas data (RNA, version 2)"
author: "Ramon Massoni-Badosa & Federico Marini"
date: "`r Sys.Date()`"
output:
  html_document:
    toc: true
    toc_float: true
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
options(width = 100)
```


# Introduction

Here we will download and process the version 2 of the [Seurat](https://zenodo.org/record/8373756) objects associated with the single-cell transcriptomes of the tonsil cell atlas to prepare them for submission to [`ExperimentHub`](https://bioconductor.org/packages/release/bioc/html/ExperimentHub.html). We will follow a similar style and structure as the one used for the [`TabulaMurisSenisData`](https://github.com/fmicompbio/TabulaMurisSenisData) package.



# Loading required packages

```{r packages}
library(Seurat)
library(SingleCellExperiment)
library(scater)
library(HDF5Array)
library(rhdf5)
library(here)
library(glue)
library(readr)
library(dplyr)
library(stringr)
library(purrr)
library(tools)
```


# Downloading the Seurat objects from Zenodo

The Seurat objects derived from our tonsil atlas have been archived as .rds files in Zenodo under the record [8373756](https://zenodo.org/record/8373756). To generate the URL's for each dataset we used the [zenodo_get](https://github.com/dvolgyes/zenodo_get) package as follows:

```{bash}
zenodo_get 10.5281/zenodo.8373756 -w tmp.txt
paste tmp.txt md5sums.txt | sed 's/\t/,/' | sed 's/ \{1,\}/,/g' > tonsil-atlas-zenodo-get-output_v2.csv
```


Let us read and curate the "tonsil-atlas-zenodo-get-output_v2.csv":

```{r}
# Read data
path_to_csv1 <- here("inst/scripts/tonsil-atlas-zenodo-get-output_v2.csv")
files_df <- read.csv(file = path_to_csv1, header = FALSE)
colnames(files_df) <- c("url", "file_id", "file_name")
```

We proceed to download the RNA dataset:

```{r}
# Create data directories
data_dir <- here("inst/scripts/rna_raw_data")
out_dir <- here("inst/scripts/HCATonsilData/2.0/RNA")
dir.create(data_dir, recursive = TRUE, showWarnings = TRUE)
dir.create(out_dir, recursive = TRUE, showWarnings = TRUE)


# Get URL and download
options(timeout = 10000000)
rna_url <- files_df[str_detect(files_df$file_name, "RNA"), "url"]
out_file <- file.path(
  data_dir,
  files_df[str_detect(files_df$file_name, "RNA"), "file_name"]
)
download.file(url = rna_url, destfile = out_file)


# Check MD5
md5_original <- files_df[str_detect(files_df$file_name, "RNA"), "file_id"]
md5_download <- md5sum(out_file)
if (!md5_original == md5_download) {
  stop("The file is corrupted!")
}


# Uncompress
untar(out_file, exdir = data_dir)
```


# Process Seurat objects to extract relevant slots

```{r}
# Get gene information
genes_df <- read_tsv(
  here("inst/extdata/features.tsv.gz"),
  col_names = c("ensembl_id", "symbol", "type")
) %>%
  filter(type == "Gene Expression") %>%
  as.data.frame()
gene_ids <- genes_df$ensembl_id
names(gene_ids) <- genes_df$symbol


# Process
files_seurat <- list.files(file.path(data_dir, "scRNA-seq"), pattern = "rds")
files_seurat <- str_subset(files_seurat, "atac", negate = TRUE)
cell_types <- files_seurat %>%
  str_split("_seurat_obj.rds") %>%
  map_chr(1) %>%
  str_remove("(20230911_|20220215_)")
cell_types[cell_types == "tonsil_atlas_rna"] <- "All"
cell_types <- str_replace(cell_types, "_", "-")
names(files_seurat) <- cell_types
for (cell_type in cell_types) {
  print(cell_type)
  
  # Read Seurat object
  file_path <- file.path(data_dir, "scRNA-seq", files_seurat[cell_type])
  seurat <- readRDS(file_path)
  dataset <- "RNA"
  
  # Extract and save reductions
  if ("pca" %in% names(seurat@reductions)) {
    tmpmat <- Embeddings(seurat, "pca")
    path_to_save_dims <- file.path(out_dir, glue("{cell_type}_{dataset}_pca.rds"))
    saveRDS(tmpmat, path_to_save_dims)
  }
  if ("harmony" %in% names(seurat@reductions)) {
    tmpmat <- Embeddings(seurat, "harmony")
    path_to_save_dims <- file.path(out_dir, glue("{cell_type}_{dataset}_harmony.rds"))
    saveRDS(tmpmat, path_to_save_dims)
  }
  if ("umap" %in% names(seurat@reductions)) {
    tmpmat <- Embeddings(seurat, "umap")
    path_to_save_dims <- file.path(out_dir, glue("{cell_type}_{dataset}_umap.rds"))
    saveRDS(tmpmat, path_to_save_dims)
  }
  
  # Extract and save rowData and colData
  saveRDS(
    as(seurat@meta.data, "DataFrame"),
    file.path(out_dir, glue("{cell_type}_{dataset}_coldata.rds"))
  )
  rowdata <- data.frame(gene_name = rownames(seurat))
  rowdata$highly_variable <- ifelse(
    rowdata$gene %in% rownames(seurat[["RNA"]]@scale.data),
    TRUE,
    FALSE
  )
  rowdata$gene_id <- gene_ids[rowdata$gene_name]
  rowdata <- as(rowdata, "DataFrame")
  rownames(rowdata) <- rowdata$gene_name
  saveRDS(rowdata, file.path(out_dir, glue("{cell_type}_{dataset}_rowdata.rds")))
  
  # Extract and save raw and processed counts
  cts <- seurat[["RNA"]]@counts
  h5_file <- file.path(out_dir, glue("{cell_type}_{dataset}_counts.h5"))
  if (!file.exists(h5_file)) {
    mat_h5 <- writeHDF5Array(
      cts,
      filepath = h5_file,
      name = "counts",
      chunkdim = HDF5Array::getHDF5DumpChunkDim(dim(cts))
    )
  }
  mat <- seurat[["RNA"]]@data
  h5_file <- file.path(out_dir, glue("{cell_type}_{dataset}_processed.h5"))
  if (!file.exists(h5_file)) {
    mat_h5 <- writeHDF5Array(
      mat,
      filepath = h5_file,
      name = "processed",
      chunkdim = HDF5Array::getHDF5DumpChunkDim(dim(mat))
    )
  }
  
  # Remove Seurat object
  rm(seurat)
  gc()
}
```


# Sanity checks

As a sanity check, let us ensure that we can create successfully each SingleCellExperiment object from its individual slots:

```{r}
all_files <- list.files(out_dir, full.names = TRUE)
for (cell_type in cell_types) {
  print(cell_type)
  path_to_cts <- str_subset(all_files, glue("{cell_type}.*counts.h5"))
  path_to_processed <- str_subset(all_files, glue("{cell_type}.*processed.h5"))
  path_to_rowdata <- str_subset(all_files, glue("{cell_type}.*rowdata.rds"))
  path_to_coldata <- str_subset(all_files, glue("{cell_type}.*coldata.rds"))
  path_to_pca <- str_subset(all_files, glue("{cell_type}.*pca.rds"))
  path_to_umap <- str_subset(all_files, glue("{cell_type}.*umap.rds"))
  path_to_harmony <-str_subset(all_files, glue("{cell_type}.*harmony.rds"))
  counts <- HDF5Array::HDF5Array(path_to_cts, name = "counts")
  processed <- HDF5Array::HDF5Array(path_to_processed, name = "processed")
  row_data <- readRDS(path_to_rowdata)
  col_data <- readRDS(path_to_coldata)
  pca <- readRDS(path_to_pca)
  umap <- readRDS(path_to_umap)
  harmony <- readRDS(path_to_harmony)
  sce <- SingleCellExperiment::SingleCellExperiment(
    assays = list(counts = counts, logcounts = processed),
    colData = col_data,
    rowData = row_data
  )
  SingleCellExperiment::reducedDims(sce) <- list(
    PCA = pca,
    UMAP = umap,
    HARMONY = harmony
  )
  print(sce)
  #print(scater::plotUMAP(sce, colour_by = "annotation_20220215"))
  if (class(sce) != "SingleCellExperiment") {
    stop(glue("The output of {cell_type} is not a SingleCellExperiment!"))
  }
}
```


# Session Information

```{r}
sessionInfo()
```

---
title: "HCATonsilData"
author: "Ramon Massoni-Badosa, Federico Marini, Alan O'Callaghan & Helena Crowell"
date: "`r BiocStyle::doc_date()`"
output: 
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{HCATonsilData}
  %\VignetteEncoding{UTF-8}
  %\VignettePackage{HCATonsilData}
  %\VignetteKeywords{GeneExpression, RNASeq, SingleCell, ATACSeq, Spatial, CITESeq}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: inline
bibliography: HCATonsilData.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = TRUE
)
```


# Introduction

Palatine tonsils are under constant exposure to antigens via the upper
respiratory tract, which makes them a compelling model secondary lymphoid organ
(SLO) to study the interplay between innate and adaptive immune cells
[@ruddle2009secondary]. Tonsils have a non-keratinizing stratified squamous
epithelium, organized into tubular, branched crypts that enlarge the tonsillar
surface. Within the crypts, microfold cells (or M cells) sample antigens at
their apical membrane. Subsequently, antigen presenting cells (APC), such as
dendritic cells (DC), process and present antigens to T cells in the
interfollicular or T cell zone. Alternatively, antigens are kept intact by
follicular dendritic cells (FDC) in lymphoid follicles, where they are
recognized by B cells [@nave2001morphology]. Such recognition triggers the
GC reaction, whereby naive B cells (NBC) undergo clonal selection,
proliferation, somatic hypermutation, class switch recombination (CSR) and
differentiation into long-lived plasma cells (PC) or memory B cells (MBC)
[@de2015dynamics].

In the context of the Human Cell Atlas (HCA) [@regev2018human], we have created 
a taxonomy of 121 cell types and states in a human tonsil. Since the transcriptome
is just a snapshot of a cell's state [@wagner2016revealing], we have added other
layers to define cell identity: single-cell resolved open chromatin epigenomic
landscapes (scATAC-seq and scRNA/ATAC-seq; i.e. Multiome) as well as protein
(CITE-seq), adaptive repertoire (single-cell B and T cell receptor sequencing;
i.e. scBCR-seq and scTCR-seq) and spatial transcriptomics (ST) profiles.

The HCATonsilData package aims to provide programmatic and modular access to the
datasets of the different modalities and cell types of the tonsil atlas.
HCATonsilData also documents how the dataset was generated and archived in
different repositories, from raw fastq files to processed datasets. It also
explains in detail the cell- and sample-level metadata, and allows users to
traceback the annotation of each cell type through detailed Glossary.


# Installation

HCATonsilData is available in BioConductor and can be installed as follows:

```{r eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("HCATonsilData")
```


Load the necessary packages in R:

```{r message=FALSE}
library("HCATonsilData")
library("SingleCellExperiment")
library("ExperimentHub")
library("scater")
library("ggplot2")
library("knitr")
library("kableExtra")
library("htmltools")
```


# Overview of the dataset

We obtained a total of 17 human tonsils, which covered three age groups:
children (n=6, 3-5 years), young adults (n=8, 19-35 years), and old adults
(n=3, 56-65 years). We collected them in a discovery cohort (n=10), which we
used to cluster and annotate cell types; and a validation cohort (n=7), 
which we used to validate the presence, annotation and markers of the discovered
cell types. The following table corresponds to the donor-level metadata:

```{r}
data("donor_metadata")

kable(
  donor_metadata,
  format = "markdown",
  caption = "Donor Metadata",
  align = "c"
) |> kable_styling(full_width = FALSE)
```


These tonsil samples were processed with different data modalities:

* scRNA-seq ([10X Chromium v3](https://www.10xgenomics.com/products/single-cell-gene-expression))
* scATAC-seq ([10X Chromium](https://www.10xgenomics.com/products/single-cell-atac))
* [10X Multiome](https://www.10xgenomics.com/products/single-cell-multiome-atac-plus-gene-expression): joint RNA and ATAC for each cell.
* [CITE-seq](https://www.nature.com/articles/nmeth.4380): joint transcriptome + ~200 protein surface markers for each cell.
* [Spatial transcriptomics]([10X Visium](https://www.10xgenomics.com/products/spatial-gene-expression))

The following heatmap informs about which samples were sequenced with which
technology and cohort type:

```{r, echo=FALSE, out.width = "100%"}
knitr::include_graphics("tonsil_atlas_cohort.png")
```


For each data modality, we mapped all raw fastq files with the appropriate flavor
of [cellranger](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/what-is-cell-ranger)
(e.g. cellranger-atac, spaceranger, etc), and analyzed all datasets within the
[Seurat](https://satijalab.org/seurat/) ecosystem [@hao2021integrated]. All
fastq files are deposited at EGA under accession number [EGAS00001006375](https://ega-archive.org/studies/EGAS00001006375).
The expression and accessibility matrices are deposited in Zenodo under record [6678331](https://zenodo.org/record/6678331).
The Seurat objects from which the data of this repository is derived are archived in Zenodo under record [8373756](https://zenodo.org/record/8373756).
Finally, the GitHub repository [Single-Cell-Genomics-Group-CNAG-CRG/TonsilAtlas](https://github.com/Single-Cell-Genomics-Group-CNAG-CRG/TonsilAtlas) documents the full
end-to-end analysis, from raw data to final Seurat objects and figures in the paper.

To define the tonsillar cell types, we first aimed to cluster cells using
the combined expression data from scRNA-seq, Multiome and CITE-seq datasets.
However, we noticed that with CITE-seq we detected 2.75X and 2.98X fewer genes
than with scRNA-seq and Multiome, respectively:


```{r, echo=FALSE, out.width = "100%"}
knitr::include_graphics("tonsil_atlas_n_detected_genes.png")
```

Because we did not want to bias the clustering towards the modality that
provides less information, we first analyzed scRNA-seq and Multiome together,
and used CITE-seq to validate the annotation (see below). However, a recent
benchmarking effort showed that scRNA-seq and single-nuclei RNA-seq (such as
Multiome) mix very poorly [@mereu2020benchmarking]. Indeed, we observed massive
batch effects between both modalities (see manuscript). To overcome it, we
found highly variable genes (HVG) for each modality independently. We then
intersected both sets of HVG to remove modality-specific variation. Following
principal component analysis (PCA), we integrated scRNA-seq and Multiome with
Harmony [@korsunsky2019fast]. Harmony scales well to atlas-level datasets
and ranks among the best-performing tools in different benchmarks.

These steps are performed using [SLOcatoR](https://github.com/massonix/SLOcatoR)
(see methods of the paper), which allowed 1. to remove batch effects between
scRNA-seq and Multiome, 2. to transfer cell type labels from Multiome to scATAC-seq,
and from scRNA-seq to CITE-seq. Thus, this strategy allowed us to annotate clusters
using information across multiple modalities (RNA expression, protein expression,
TF motifs, etc.).

Note: we provide SLOcatoR as an R package to promote reproducibility and
best practices in scientific computing, but we refer the users to properly
benchmarked tools to annotate their datasets such as
[Azimuth](https://www.cell.com/cell/fulltext/S0092-8674%2821%2900583-3),
[scArches](https://www.nature.com/articles/s41587-021-01001-7),
or [Symphony](https://www.nature.com/articles/s41467-021-25957-x).


Within the discovery cohort, we first defined 9 main compartments: (1) NBC and MBC,
(2) GC B cells (GCBC), (3) PC, (4) CD4 T cells, (5) cytotoxic cells
[CD8 T cells, NK, ILC, double negative T cells (DN)], (6) myeloid cells
(DC, macrophages, monocytes, granulocytes, mast cells), (7) FDC, (8) epithelial
cells and (9) plasmacytoid dendritic cells (PDC):

```{r, echo=FALSE, out.width = "100%"}
knitr::include_graphics("tonsil_atlas_umap_9_compartments.png")
```

Subsequently, we clustered and annotated cell subtypes within each of the main
compartments, giving rise to the 121 cell types and states in the atlas. The
following UMAPs inform about the main cell populations and number of cells
per assay in the discovery cohort:

```{r, echo=FALSE, out.width = "100%"}
knitr::include_graphics("tonsil_atlas_umaps.png")
```

We then applied SLOcatoR to annotate cells from the validation cohort (1) to confirm
the presence, annotation and markers of cell types; (2) to extend the atlas
through an integrated validation dataset, and (3) to chart compositional
changes in the tonsil during aging. The integrated tonsil atlas represented
over 462,000 single-cell transcriptomes:

```{r, echo=FALSE, out.width = "100%"}
knitr::include_graphics("tonsil_atlas_umap_discovery_validation_cohort.png")
```

Finally, we focused again on each of the 9 compartments and retransferred the
final annotation level. As an example, here we can see the transferred annotation
for CD4 T cells, from discovery to validation cohort:

```{r, echo=FALSE, out.width = "100%"}
knitr::include_graphics("tonsil_atlas_umap_discovery_validation_cohort_CD4.png")
```

Overall, we could validate most of the cell types, as they preserved their
marker genes and had a high annotation confidence in the validation cohort.
However, (1) transient cell states, (2) underrepresented cell types, or
(3) clusters that are at the intersection between two main compartments (and
might represent doublets) had low annotation confidences. We provide this
interpretation in the Glossary (see below).


# Assay types

HCATonsilData provides access to 5 main types of assays: RNA, ATAC, Multiome, 
CITE-seq and Spatial.


## scRNA-seq

We can obtain the `SingleCellExperiment` object with gene expression (RNA)
data as follows:

```{r eval=FALSE}
(sce <- HCATonsilData(assayType = "RNA", cellType = "All"))
table(sce$assay)
```

This object consists of 377,988 profiled with scRNA-seq (3P)
and 84,364 cells profiled with multiome, for a total of 462,352 cells (37,378
genes were quantified across all of these).

We can dowload a `SingleCellExperiment` object specific to each of the main 
subpopulations defined at level 1 as follows:

```{r}
listCellTypes(assayType = "RNA")
(epithelial <- HCATonsilData(assayType = "RNA", cellType = "epithelial"))
data("colors_20230508")
scater::plotUMAP(epithelial, colour_by = "annotation_20230508") +
  ggplot2::scale_color_manual(values = colors_20230508$epithelial) +
  ggplot2::theme(legend.title = ggplot2::element_blank())
```

As we can see, we can use the same colors as in the publication using the
`colors_20230508` variable.

We can also download datasets from the preprint (discovery cohort, version 1.0) - 
we first load a list of matched annotations used throughout the course of the 
project, used internally by the main `HCATonsilData()` function:

```{r}
data("annotations_dictionary")
annotations_dictionary[["dict_20220619_to_20230508"]]

# load also a predefined palette of colors, to match the ones used in the manuscript 
data("colors_20230508")

(epithelial_discovery <- HCATonsilData("RNA", "epithelial", version = "1.0"))
scater::plotUMAP(epithelial, colour_by = "annotation_20230508") +
  ggplot2::scale_color_manual(values = colors_20230508$epithelial) +
  ggplot2::theme(legend.title = ggplot2::element_blank())
```

Here's a brief explanation of all the variables in the colData slot of the
`SingleCellExperiment` objects:


* barcode: the cell barcode. Combination of [GEM well](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/glossary) and cellranger 10X barcode.
* donor_id: the donor-specific identifier, which can be used to retrieve donor-level metadata form the table above.
* gem_id: we gave a unique hashtag to each [GEM well](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/glossary) (10X Chip Channel) in our dataset. This allows to traceback all the metadata for a given cell.       
* library_name: each GEM well can give rise to multiple Illumina libraries. For example, one Multiome GEM well will give rise to 2 illumina libraries (ATAC and RNA).
* assay: 3P (scRNA-seq) or multiome
* sex
* age
* age_group: kid, young adult, old adult
* hospital: hospital where the tonsils where obtained [Hospital Clinic](https://www.clinicbarcelona.org/), [CIMA](https://cima.cun.es/), or Newcastle.
* cohort_type: discovery or validation cohort
* cause_for_tonsillectomy: condition that led to the tonsillectomy (e.g. tonsillitis, sleep apnea, etc.)
* is_hashed: whether we used [cell hashing](https://genomebiology.biomedcentral.com/articles/10.1186/s13059-018-1603-1) or not.
* preservation: whether we processed the sample fresh or frozen.
* nCount_RNA, nFeature_RNA: number of counts and features (genes) detected per cell.
* pct_mt, pct_ribosomal: percentage of counts that map to mitochondrial (^MT) or ribosomal (^RPS) genes.
* pDNN_hashing, pDNN_scrublet, pDNN_union: proportion of doublet nearest neighbors (pDNN) using different doublet annotations.
* S.Score, G2M.Score Phase, CC.Difference: outputs of the [CellCycleScoring](https://rdrr.io/cran/Seurat/man/CellCycleScoring.html) from `Seurat`.
* scrublet_doublet_scores: doublet score obtained with [Scrublet](https://www.cell.com/cell-systems/fulltext/S2405-4712(18)30474-5?_returnURL=https%3A%2F%2Flinkinghub.elsevier.com%2Fretrieve%2Fpii%2FS2405471218304745%3Fshowall%3Dtrue)
* scrublet_predicted_doublet: doublet annotation obtained with Scirpy (doublet or singlet)
* doublet_score_scDblFinder: doublet score obtained with [scDblFinder](https://f1000research.com/articles/10-979), which was run in the validation cohort following [the most recent best practices for single-cell analysis](https://www.nature.com/articles/s41576-023-00586-w).
* annotation_level_1: annotation used at our level 1 clustering (9 main compartments).
* annotation_level_1_probability: annotation confidence for the level 1 annotation (relevant for validation cohort, as it implied KNN annotation)
* annotation_figure_1: annotation used in the figure 1 of our manuscript. This annotation consisted of grouping the final subtypes into main cell types that were distinguishable in the UMAP.
* annotation_20220215, annotation_20220619, annotation_20230508: time-stamped annotation for cell types.
* annotation_20230508_probability: annotation confidence for the final annotation (relevant for validation cohort, as it implied KNN annotation)
* UMAP_1_level_1, UMAP_2_level_1: UMAP1 coordinates of the figure 1B of the article.
* annotation_20220215: see above.
* UMAP_1_20220215, UMAP_2_20220215: UMAP coordinates used in figures of the preprint for each cell type.


## scATAC-seq and Multiome

Since there is not a popular Bioconductor package to analyze or store scATAC-seq data,
we point users to the scATAC-seq and Multiome Seurat objects that we created
using [Signac](https://stuartlab.org/signac/) [@stuart2021single].

Here are the instructions to download the scATAC-seq object (approximately ~9.3 Gb in size):

```{r eval=FALSE}
library("Seurat")
library("Signac")

download_dir = tempdir()

options(timeout = 10000000)
atac_url <- "https://zenodo.org/record/8373756/files/TonsilAtlasSeuratATAC.tar.gz"
download.file(
  url = atac_url, 
  destfile = file.path(download_dir, "TonsilAtlasSeuratATAC.tar.gz")
)
# Advice: check that the md5sum is the same as the one in Zenodo
untar(
  tarfile = file.path(download_dir, "TonsilAtlasSeuratATAC.tar.gz"), 
  exdir = download_dir
)
atac_seurat <- readRDS(
  file.path(download_dir, "scATAC-seq/20230911_tonsil_atlas_atac_seurat_obj.rds")
)
atac_seurat
```


The multiome object contains 68,749 cells that passed both RNA and ATAC QC filters.
Here are the instructions to download the Multiome object (approximately ~5.7 Gb in size):

```{r eval=FALSE}
library("Seurat")
library("Signac")

download_dir = tempdir()

options(timeout = 10000000)
multiome_url <- "https://zenodo.org/record/8373756/files/TonsilAtlasSeuratMultiome.tar.gz"
download.file(
  url = multiome_url, 
  destfile = file.path(download_dir, "TonsilAtlasSeuratMultiome.tar.gz")
)
# Advice: check that the md5sum is the same as the one in Zenodo
untar(
  tarfile = file.path(download_dir, "TonsilAtlasSeuratMultiome.tar.gz"), 
  exdir = download_dir
)
multiome_seurat <- readRDS(
  file.path(download_dir, "/multiome/20230911_tonsil_atlas_multiome_seurat_obj.rds")
)
multiome_seurat
```


**Note**: we may version these two objects in Zenodo to polish the annotation.
Therefore, we recommend users to check that this is the most updated version
of the object in Zenodo.

To recall peaks or visualize chromatin tracks, it is essential to have access to
the [fragments file](https://support.10xgenomics.com/single-cell-atac/software/pipelines/latest/output/fragments).
We modified the original fragments files generated by
cellranger-atac or cellranger-arc to include the [gem_id](https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/glossary)
as prefix. These files can be downloaded as follows (approximate size of ~26.0 Gb, feel free to grab a coffee in the meanwhile):

```{r eval=FALSE}
download_dir = tempdir()

fragments_url <- "https://zenodo.org/record/8373756/files/fragments_files.tar.gz"
download.file(
  url = fragments_url, 
  destfile = file.path(download_dir, "fragments_files.tar.gz")
)
untar(
  tarfile = file.path(download_dir, "fragments_files.tar.gz"),
  exdir = download_dir
)
```

**Note**: it is paramount to update the paths to the fragments files in each
ChromatinAssay object using the [UpdatePath](https://stuartlab.org/signac/reference/updatepath)
function.

The chromatin accessibility side of these objects was processed using the
general pipeline from [Signac](https://stuartlab.org/signac/articles/pbmc_vignette). Briefly:

* We called peaks using [MACS2](https://pypi.org/project/MACS2/) (as implemented in the [CallPeaks](https://www.rdocumentation.org/packages/Signac/versions/1.10.0/topics/CallPeaks) function), generating pseudobulks for each of the main 9 compartments.
* We quantified the accessibility for each peak and each cell using the [FeatureMatrix](https://stuartlab.org/signac/reference/featurematrix) function.
* We normalized data using the term frequency inverse document frequency (TF-IDF) normalization, as implemented in the [RunTFIDF](https://stuartlab.org/signac/reference/runtfidf) function.
* We reduced the dimensionality using latent semantic indexing ([RunLSI](https://www.rdocumentation.org/packages/Seurat/versions/3.1.4/topics/RunLSI) function)

For Multiome, we additionally computed a weighted KNN graph with the
[FindMultiModalNeighbors](https://satijalab.org/seurat/reference/findmultimodalneighbors) function.

Many variables in the metadata slot are shared with scRNA-seq. Here, we explain
the meaning of the variables that are specific to Multiome or scATAC-seq:

* nCount_ATAC: total number of counts per cell
* nFeature_ATAC: total number of peaks detected per cell
* is_facs_sorted: whether that particular sample was FACS-sorted to eliminate dead cells
* nucleosome_signal: as explained in Signac, *Nucleosome banding pattern: The histogram of DNA fragment sizes (determined from the paired-end sequencing reads) should exhibit a strong nucleosome banding pattern corresponding to the length of DNA wrapped around a single nucleosome. We calculate this per single cell, and quantify the approximate ratio of mononucleosomal to nucleosome-free fragments*
* nucleosome_percentile: percentile rank of each ratio of mononucleosomal to nucleosome-free fragments per cell (see [NucleosomeSignal](https://stuartlab.org/signac/reference/nucleosomesignal) function)
* TSS.enrichment: as explained in Signac, *The ENCODE project has defined an ATAC-seq targeting score based on the ratio of fragments centered at the TSS to fragments in TSS-flanking regions (see https://www.encodeproject.org/data-standards/terms/). Poor ATAC-seq experiments typically will have a low TSS enrichment score.*
* TSS.percentile
* RNA.weight: the weight that RNA has in the weighted nearest neighbor graph per cell (multiome-specific)
* ATAC.weight: the weight that RNA has in the weighted nearest neighbor graph per cell (multiome-specific)


## CITE-seq

The CITE-seq object can be downloaded as follows (approximate size of ~0.4 Gb):

```{r eval=FALSE}
library("Seurat")

download_dir = tempdir()

options(timeout = 10000000)
cite_url <- "https://zenodo.org/record/8373756/files/TonsilAtlasSeuratCITE.tar.gz"
download.file(
  url = cite_url, 
  destfile = file.path(download_dir, "TonsilAtlasSeuratCITE.tar.gz")
)
# Advice: check that the md5sum is the same as the one in Zenodo
untar(
  tarfile = file.path(download_dir, "TonsilAtlasSeuratCITE.tar.gz"), 
  exdir = download_dir
)

cite_seurat <- readRDS(
  file.path(download_dir, "CITE-seq/20220215_tonsil_atlas_cite_seurat_obj.rds")
)
cite_seurat
```


Together with CITE-seq, we also applied scBCR-seq and scTCR-seq. This data was
analyzed with [scirpy](https://academic.oup.com/bioinformatics/article/36/18/4817/5866543),
[@sturm2020scirpy] and we provide the main output of this analysis (for cytotoxic cells) as a
dataframe, which can be imported as follows:

```{r eval=FALSE}
scirpy_df <- read.csv(
  file = file.path(download_dir, "CITE-seq/scirpy_tcr_output.tsv"),
  header = TRUE
)

head(scirpy_df)
```


## Spatial transcriptomics

A `r BiocStyle::Biocpkg("SpatialExperiment")` of the [spatial transcriptomics]([10X Visium](https://www.10xgenomics.com/products/spatial-gene-expression)) dataset may be retrieved via `assayType="Spatial"`. The dataset contains 8 tissue slices with ~1,000-3,000 cells each that were profiled on two separated slides, as well as a low-resolution (H&E staining) image for each slice.

```{r eval=FALSE}
library("SpatialExperiment")
(spe <- HCATonsilData(assayType = "Spatial"))
```


To plot gene expression you can use the ggspavis package:

```{r eval=FALSE}
library("ggspavis")
sub <- spe[, spe$sample_id == "esvq52_nluss5"]
plt <- plotVisium(sub, fill="SELENOP") + 
  scale_fill_gradientn(colors=rev(hcl.colors(9, "Spectral")))
plt$layers[[2]]$aes_params$size <- 1.5
plt$layers[[2]]$aes_params$alpha <- 1
plt$layers[[2]]$aes_params$stroke <- NA
plt
```


# Annotations of the Tonsil Data Atlas

To allow users to traceback the rationale behind each and every of our annotations,
we provide a detailed glossary of 121 cell types and states and related functions
to get the explanation, markers and references of every annotation. You can access
the glossary as a dataframe as follows:


```{r}
glossary_df <- TonsilData_glossary()
head(glossary_df)
```


To get the glossary for each cell type with nice printing formatting you can use
the `TonsilData_cellinfo()` function:

```{r}
TonsilData_cellinfo("Tfr")
```


Alternatively, you can get a static html with links to markers and articles with
`TonsilData_cellinfo_html`

```{r eval = TRUE}
htmltools::includeHTML(
  TonsilData_cellinfo_html("Tfr", display_plot = TRUE)
)
```


Although we have put massive effort in annotating tonsillar cell types, cell type
annotations are dynamic by nature. New literature or other interpretations of the
data can challenge and refine our annotations. To accommodate this, we have developed
the `updateAnnotation` function, which allows to periodically provide newer
annotations as extra columns in the `colData` slot of the `SingleCellExperiment`
objects. If you want to contribute in one of these versions of the upcoming annotations,
please [open an issue](https://github.com/massonix/HCATonsilData/issues/new) and
describe your annotation.


# Interoperability with other frameworks

While we provide data in the form of SingleCellExperiment objects, you may want to
analyze your data using a different single-cell data container. In future releases,
we will strive to make the SingleCellExperiment objects compatible with
[Seurat v5](https://github.com/satijalab/seurat), which will come out after this
release of BioConductor. Alternatively, you may want to obtain
[AnnData](https://anndata.readthedocs.io/en/latest/) objects to analyze your data
in [scanpy](https://scanpy.readthedocs.io/en/stable/installation.html) ecosystem.
You can convert and save the data as follows:

```{r eval=FALSE}
library("zellkonverter")
epithelial <- HCATonsilData(assayType = "RNA", cellType = "epithelial")

epi_anndatafile <- file.path(tempdir(), "epithelial.h5ad")

writeH5AD(sce = epithelial, file = epi_anndatafile)
```

The SingleCellExperiment objects obtained via `HCATonsilData()` can be explored in 
detail using e.g. additional Bioconductor packages, such as the `iSEE` package.

This can be as simple as executing this chunk:

```{r launchisee, eval=FALSE}
if (require(iSEE)) {
  iSEE(epithelial)
}
```
 
# Session information {-}

```{r}
sessionInfo()
```


# References

---
title: "Download and preprocess the tonsil atlas data (RNA)"
author: "Ramon Massoni-Badosa, Federico Marini & Alan O'Callaghan"
date: "`r Sys.Date()`"
output:
  html_document:
    toc: true
    toc_float: true
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
options(width = 100)
```


# Introduction

Here we will download and process the Seurat objects associated with the tonsil cell atlas to prepare them for submission to [`ExperimentHub`](https://bioconductor.org/packages/release/bioc/html/ExperimentHub.html). We will follow a similar style and structure as the one used for the [`TabulaMurisSenisData`](https://github.com/fmicompbio/TabulaMurisSenisData) pacakge.


# Loading required packages

```{r packages}
library(Seurat)
library(SingleCellExperiment)
library(scater)
library(HDF5Array)
library(rhdf5)
library(here)
library(glue)
library(readr)
library(dplyr)
library(stringr)
library(tools)
```


# Downloading the Seurat objects from Zenodo

The Seurat objects derived from our tonsil atlas have been archived as .rds files in Zenodo under the record [6340174](https://zenodo.org/record/6340174). To generate the URL's for each package we used the [zenodo_get](https://github.com/dvolgyes/zenodo_get) package as follows:

```{bash}
zenodo_get 10.5281/zenodo.6340174 -w tmp.txt
paste tmp.txt md5sums.txt | sed 's/\t/,/' | sed 's/ \{1,\}/,/g' > tonsil-atlas-zenodo-get-output.csv
```


Let us read and curate the "tonsil-atlas-rds-files.csv" to include the file name, URL, cell type and dataset for each object:

```{r}
# Read data
path_to_csv1 <- here("inst/scripts/tonsil-atlas-zenodo-get-output.csv")
path_to_csv2 <- here("inst/scripts/tonsil-atlas-rds-files.csv")
files_df <- read.csv(file = path_to_csv1, header = FALSE)
colnames(files_df) <- c("url", "file_id", "file_name")


# Set cell type and dataset variables
files_df$cell_type <- files_df$file_name %>%
  str_remove("20220215_") %>%
  str_remove("_seurat_obj.*$")
files_df$cell_type[files_df$cell_type == "CD4_T"] <- "CD4-T"
files_df$cell_type[files_df$cell_type == "CD8_T"] <- "CD8-T"
files_df$cell_type[files_df$cell_type == "NBC_MBC"] <- "NBC-MBC"
files_df$cell_type[files_df$cell_type == "ILC_NK"] <- "ILC-NK"
files_df$cell_type[str_detect(files_df$cell_type, "tonsil")] <- "All"
files_df$dataset <- case_when(
  str_detect(files_df$file_name, "atac") ~ "ATAC",
  str_detect(files_df$file_name, "spatial") ~ "Spatial",
  str_detect(files_df$file_name, "cite") ~ "CITE",
  TRUE ~ "RNA"
)


# Reorder columns
new_order <- c("file_name", "file_id", "cell_type", "dataset", "url")
files_df <- files_df[, new_order]


# Rewrite
write.csv(files_df, file = path_to_csv2, row.names = FALSE, quote = FALSE)
```


Now that we have the URL we can download the files:

```{r}
# Create data directories
data_dir <- here("inst/scripts/rna_raw_data")
out_dir <- here("inst/scripts/HCATonsilData/1.0/RNA")
dir.create(data_dir, recursive = TRUE, showWarnings = TRUE)
dir.create(out_dir, recursive = TRUE, showWarnings = TRUE)


# Read and filter list of files to download
files_df <- read.csv(file = path_to_csv2, header = TRUE)
files_df <- files_df[files_df$dataset == "RNA", ]


# Downlaod
options(timeout = 1440)
for (i in seq_len(nrow(files_df))) {
  out_file <- file.path(data_dir, files_df$file_name[i])
  if (!file.exists(out_file)) {
    download.file(url = files_df$url[i], destfile = out_file)
  }
  
  # Check MD5
  md5_original <- files_df$file_id[i]
  md5_download <- md5sum(out_file)
  if (!md5_original == md5_download) {
    stop("The file is corrupted!")
  }
}
```


# Process Seurat objects to extract relevant slots

```{r}
# Get gene information
genes_df <- read_tsv(
  here("inst/extdata/features.tsv.gz"),
  col_names = c("ensembl_id", "symbol", "type")
) %>%
  filter(type == "Gene Expression") %>%
  as.data.frame()
gene_ids <- genes_df$ensembl_id
names(gene_ids) <- genes_df$symbol


# Process
for (f in files_df$file_name) {
  print(f)
  
  # Read Seurat object
  seurat <- readRDS(file.path(data_dir, f))
  cell_type <- files_df[files_df$file_name == f, "cell_type"]
  dataset <- files_df[files_df$file_name == f, "dataset"]
  
  # Extract and save reductions
  if ("pca" %in% names(seurat@reductions)) {
    tmpmat <- Embeddings(seurat, "pca")
    path_to_save_dims <- file.path(out_dir, glue("{cell_type}_{dataset}_pca.rds"))
    saveRDS(tmpmat, path_to_save_dims)
  }
  if ("harmony" %in% names(seurat@reductions)) {
    tmpmat <- Embeddings(seurat, "harmony")
    path_to_save_dims <- file.path(out_dir, glue("{cell_type}_{dataset}_harmony.rds"))
    saveRDS(tmpmat, path_to_save_dims)
  }
  if ("umap" %in% names(seurat@reductions)) {
    tmpmat <- Embeddings(seurat, "umap")
    path_to_save_dims <- file.path(out_dir, glue("{cell_type}_{dataset}_umap.rds"))
    saveRDS(tmpmat, path_to_save_dims)
  }
  
  # Extract and save rowData and colData
  saveRDS(
    as(seurat@meta.data, "DataFrame"),
    file.path(out_dir, glue("{cell_type}_{dataset}_coldata.rds"))
  )
  rowdata <- data.frame(gene_name = rownames(seurat))
  rowdata$highly_variable <- ifelse(
    rowdata$gene %in% rownames(seurat[["RNA"]]@scale.data),
    TRUE,
    FALSE
  )
  rowdata$gene_id <- gene_ids[rowdata$gene_name]
  rowdata <- as(rowdata, "DataFrame")
  rownames(rowdata) <- rowdata$gene_name
  saveRDS(rowdata, file.path(out_dir, glue("{cell_type}_{dataset}_rowdata.rds")))

  # Extract and save raw and processed counts
  cts <- seurat[["RNA"]]@counts
  h5_file <- file.path(out_dir, glue("{cell_type}_{dataset}_counts.h5"))
  if (!file.exists(h5_file)) {
    mat_h5 <- writeHDF5Array(
      cts,
      filepath = h5_file,
      name = "counts",
      chunkdim = HDF5Array::getHDF5DumpChunkDim(dim(cts))
    )
  }
  mat <- seurat[["RNA"]]@data
  h5_file <- file.path(out_dir, glue("{cell_type}_{dataset}_processed.h5"))
  if (!file.exists(h5_file)) {
    mat_h5 <- writeHDF5Array(
      mat,
      filepath = h5_file,
      name = "processed",
      chunkdim = HDF5Array::getHDF5DumpChunkDim(dim(mat))
    )
  }
  
  # Remove Seurat object
  rm(seurat)
  gc()
}
```


# Sanity checks

As a sanity check, let us ensure that we can create successfully each SingleCellExperiment object from its individual slots:

```{r}
all_files <- list.files(out_dir, full.names = TRUE)
for (cell_type in files_df$cell_type) {
  print(cell_type)
  path_to_cts <- str_subset(all_files, glue("{cell_type}.*counts.h5"))
  path_to_processed <- str_subset(all_files, glue("{cell_type}.*processed.h5"))
  path_to_rowdata <- str_subset(all_files, glue("{cell_type}.*rowdata.rds"))
  path_to_coldata <- str_subset(all_files, glue("{cell_type}.*coldata.rds"))
  path_to_pca <- str_subset(all_files, glue("{cell_type}.*pca.rds"))
  path_to_umap <- str_subset(all_files, glue("{cell_type}.*umap.rds"))
  path_to_harmony <-str_subset(all_files, glue("{cell_type}.*harmony.rds"))
  counts <- HDF5Array::HDF5Array(path_to_cts, name = "counts")
  processed <- HDF5Array::HDF5Array(path_to_processed, name = "processed")
  row_data <- readRDS(path_to_rowdata)
  col_data <- readRDS(path_to_coldata)
  pca <- readRDS(path_to_pca)
  umap <- readRDS(path_to_umap)
  sce <- SingleCellExperiment::SingleCellExperiment(
    assays = list(counts = counts, logcounts = processed),
    colData = col_data,
    rowData = row_data
  )
  SingleCellExperiment::reducedDims(sce) <- list(
    PCA = pca,
    UMAP = umap
  )
  if (length(path_to_harmony) == 1) {
    harmony <- readRDS(path_to_harmony)
    SingleCellExperiment::reducedDim(sce, "HARMONY") <- harmony
  }
  print(sce)
  print(scater::plotUMAP(sce, colour_by = "annotation_20220215"))
  if (class(sce) != "SingleCellExperiment") {
    stop(glue("The output of {cell_type} is not a SingleCellExperiment!"))
  }
}
```


# Session Information

```{r}
sessionInfo()
```

---
title: "Download and preprocess the tonsil atlas data (Visium)"
author: "Ramon Massoni-Badosa, Marc Elosua-Bayes & Helena Lucia Crowell"
date: "`r Sys.Date()`"
output:
  html_document:
    toc: true
    toc_float: true
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
options(width = 100)
```

# Introduction

Here we will download and process the Seurat objects associated with the tonsil cell atlas to prepare them for submission to [`ExperimentHub`](https://bioconductor.org/packages/release/bioc/html/ExperimentHub.html). We will follow a similar style and structure as the one used for the [`TabulaMurisSenisData`](https://github.com/fmicompbio/TabulaMurisSenisData) pacakge.

In particular, here we will focus on the [Visium](https://www.10xgenomics.com/products/spatial-gene-expression) data from our tonsil atlas.

# Loading required packages

```{r dependencies}
library(here)
library(glue)
library(Seurat)
library(HDF5Array)
library(stringr)
library(SpatialExperiment)
library(tools)
library(dplyr)
library(readr)
library(purrr)
library(tibble)
```


# Downloading the Seurat objects from Zenodo

<!--
The Seurat objects derived from our tonsil atlas have been archived as .rds files in Zenodo under the record [8373756](https://zenodo.org/record/8373756). To generate the URL's for each dataset we used the [zenodo_get](https://github.com/dvolgyes/zenodo_get) package as follows:

```{bash}
zenodo_get 10.5281/zenodo.8373756 -w tmp.txt
paste tmp.txt md5sums.txt | sed 's/\t/,/' | sed 's/ \{1,\}/,/g' > tonsil-atlas-zenodo-get-output_v2.csv
```
-->

The chunk above was run already to download the scRNA-seq data in another notebook, so we can read the "tonsil-atlas-zenodo-get-output_v2.csv" file and get the URL of the Visium data:

```{r}
# Read data
path_to_csv1 <- here("inst/scripts/tonsil-atlas-zenodo-get-output_v2.csv")
files_df <- read.csv(file = path_to_csv1, header = FALSE)
colnames(files_df) <- c("url", "file_id", "file_name")
```

We proceed to download the Visium dataset:

```{r}
# Create data directories
data_dir <- here("inst/scripts/spatial_raw_data")
out_dir <- here("inst/scripts/HCATonsilData/2.0/Spatial")
dir.create(data_dir, recursive = TRUE, showWarnings = TRUE)
dir.create(out_dir, recursive = TRUE, showWarnings = TRUE)


# Get URL and download
options(timeout = 1000000)
filt <- str_detect(files_df$file_name, "Spatial")
spatial_url <- files_df[filt, "url"]
out_file <- file.path(
  data_dir,
  files_df[filt, "file_name"]
)
download.file(url = spatial_url, destfile = out_file)


# Check MD5
md5_original <- files_df[filt, "file_id"]
md5_download <- md5sum(out_file)
if (!md5_original == md5_download) {
  stop("The file is corrupted!")
}


# Uncompress
untar(out_file, exdir = data_dir)
```


# Process Seurat objects to extract relevant slots

```{r}
# Read data
file_path <- here("inst/scripts/spatial_raw_data/spatial_transcriptomics/20220527_tonsil_atlas_spatial_seurat_obj.rds")
so <- readRDS(file_path)


# 'assays'
for (i in c("counts", "data")) {
    j <- switch(i, counts="counts", data="logcounts")
    fnm <- file.path(out_dir, glue("Spatial_assay_{j}.h5"))
    mtx <- GetAssayData(so, i)
    h5 <- writeHDF5Array(mtx, 
        filepath=fnm, name=j, 
        chunkdim=getHDF5DumpChunkDim(dim(mtx)))
}

# 'reducedDims'
for (dr in Reductions(so)) {
    obj <- Embeddings(so, dr)
    rownames(obj) <- NULL
    fnm <- file.path(out_dir, glue("Spatial_dimred_{dr}.rds"))
    saveRDS(obj, fnm)
}


# 'colData'
spatial_coordinates <- map_df(Images(so), \(.) so@images[[.]]@coordinates)
spatial_coordinates <- rownames_to_column(spatial_coordinates, "barcode")
cell_metadata <- left_join(so@meta.data, spatial_coordinates, by = "barcode")
cd <- DataFrame(cell_metadata)
cd$sample_id <- cd$gem_id # needed for SPE
saveRDS(cd, file.path(out_dir, "Spatial_coldata.rds"))


# 'rowData'
genes_df <- read_tsv(
  here("inst/extdata/features.tsv.gz"),
  col_names = c("ensembl_id", "symbol", "type")
) %>%
  filter(type == "Gene Expression") %>%
  as.data.frame()
gene_ids <- genes_df$ensembl_id
names(gene_ids) <- genes_df$symbol
gs <- rownames(so)
rd <- so[["Spatial"]]@meta.features
rd <- DataFrame(row.names=gs, gene_name=gs, rd)
rd$gene_id <- gene_ids[rd$gene_name]
names(rd$gene_id) <- NULL
rd$highly_variable <- gs %in% VariableFeatures(so)
saveRDS(rd, file.path(out_dir, "Spatial_rowdata.rds"))

# 'imgData'
id <- lapply(Images(so), \(id) {
    sfs <- so@images[[id]]@scale.factors
    img <- GetImage(so, image=id, mode="raster")
    saveRDS(img, file.path(out_dir, glue("Spatial_image_{id}.rds")))
    saveRDS(sfs, file.path(out_dir, glue("Spatial_scale_{id}.rds")))
})
```


# Sanity checks

As a sanity check, let us ensure that we can create successfully each SingleCellExperiment object from its individual slots:

```{r load}
# metadata
rd <- readRDS(file.path(out_dir, "Spatial_rowdata.rds"))
cd <- readRDS(file.path(out_dir, "Spatial_coldata.rds"))

# assays
as <- list.files(out_dir, "^Spatial_assay_", full.names=TRUE)
names(as) <- gsub("^Spatial_assay_(.*)\\.h5$", "\\1", basename(as))
as <- mapply(
    \(h5, nm) {
        mtx <- h5read(h5, name=nm)
        rownames(mtx) <- rd$gene_name
        colnames(mtx) <- cd$barcode
        as(mtx, "dgCMatrix")
    },
    h5=as, nm=names(as), SIMPLIFY=FALSE)

# reductions
dr <- list.files(out_dir, "^Spatial_dimred_", full.names=TRUE)
names(dr) <- toupper(gsub("^Spatial_dimred_(.*)\\.rds", "\\1", basename(dr)))
dr <- lapply(dr, readRDS)

# images
id <- lapply(unique(cd$sample_id), \(id) {
    img <- readRDS(file.path(out_dir, glue("Spatial_image_{id}.rds")))
    sfs <- readRDS(file.path(out_dir, glue("Spatial_scale_{id}.rds")))
    DataFrame(
        sample_id=id, image_id="lowres",
        data=I(list(SpatialImage(img))), 
        scaleFactor=sfs$lowres)
}) |> do.call(what="rbind")
```

Construct SpatialExperiment object:

```{r spe}
# construct 'SpatialExperiment'
(spe <- SpatialExperiment(assays=as, 
    mainExpName="Spatial",
    reducedDims=dr, imgData=id,
    rowData=rd, colData=cd)) #metadata=md
```

```{r check, warning=FALSE}
sce_old <- as.SingleCellExperiment(so)
sce_new <- as(spe, "SingleCellExperiment")
# drop class-specific identifiers
sce_old$ident <- sce_new$sample_id <- NULL
# drop 'spatialCoords'
int_colData(sce_new)$spatialCoords <- NULL
# match ordering of reduced dimensions
drs <- reducedDimNames(sce_old)
reducedDims(sce_new) <- reducedDims(sce_new)[drs]
# drop irrelevant internals & dimension names
imd <- c("version", "mainExpName")
int_metadata(sce_old) <- int_metadata(sce_old)[imd]
int_metadata(sce_new) <- int_metadata(sce_new)[imd]
rownames(colData(sce_old)) <- rownames(colData(sce_new)) <- NULL
# this is necessary, too...
elementMetadata(rowRanges(sce_old)) <- elementMetadata(rowRanges(sce_new))
# check saved & re-loaded objects match
identical(sce_old, sce_new)
```


# Session Information

```{r session-info}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/updateAnnotation.R
\name{updateAnnotation}
\alias{updateAnnotation}
\title{Update cell type annotation}
\usage{
updateAnnotation(sce, refAnnotation = "20220215", newAnnotation = "20220619")
}
\arguments{
\item{sce}{A SingleCellExperiment object obtained using HCATonsilData
function.}

\item{refAnnotation}{string specifying the date of the annotation to use as
reference.}

\item{newAnnotation}{string specifying the suffix to add to the new column
(annotation_*).}
}
\value{
A SingleCellExperiment object with additional an additional column
(annotation_*) that contains more annotations.
}
\description{
Annotations are dynamic by nature. As more experts look into the data and
newer literature comes out, we expect annotations to be refined over time.
We have accommodated this by allowing us and users to add new annotations
to the SingleCellExperiment objects. If you want to propose a new
annotation based on your experience or new evidence, please open an issue
at https://github.com/massonix/HCATonsilData/issues.
}
\examples{
# update the annotation from preprint (version 1.0) to publication (version 2.0)
\dontrun{
sce <- updateAnnotation(
  sce,
  refAnnotation = "20220619",
  newAnnotation = "20230508"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NBC_MBC_annotation_df}
\alias{NBC_MBC_annotation_df}
\title{Annotation dictionary for naive and memory B cells (NBC-MBC)}
\format{
A dataframe with 112478 NBC-MBC and 5 variables
\describe{
\item{barcode}{cell barcode}
\item{names_level_5_clusters_eta}{Cell annotation given by the B-cell annotation team in February 2022}
\item{annotation_20220619}{Cell annotation given by the B-cell annotation team June 2022 (preprint)}
\item{UMAP_1}{UMAP1 coordinates}
\item{UMAP_2}{UMAP2 coordinates}
}
}
\source{
\url{https://zenodo.org/record/8373756}
}
\description{
In version 1 of the atlas, we changed the clusters and annotation for NBC-MBC
prior to publishing the preprint. Since, there is not a 1:1 mapping between
clusters, here we provide the correspondence between the annotations in
February 2022 and the preprint (June 2022) for each NBC-MBC cell barcode.
}
\details{
This data is used by the updateAnnotation function in B cells.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glossary.R
\name{TonsilData_glossary}
\alias{TonsilData_glossary}
\title{TonsilData_glossary}
\usage{
TonsilData_glossary()
}
\value{
Data frame containing the info on the cell types included in the
TonsilDataAtlas
}
\description{
Convenience function to read directly in the file provided as \code{extdata}
}
\examples{
glossary_df <- TonsilData_glossary()
head(glossary_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{donor_metadata}
\alias{donor_metadata}
\title{Donor metadata}
\format{
A dataframe with 17 observations and 8 variables
}
\source{
Check the tonsil atlas publication
}
\description{
Data frame that contains all metadata information for the 17 donors included
in the tonsil atlas.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{annotations_dictionary}
\alias{annotations_dictionary}
\title{Annotation dictionary to keep track of annotations changes and versions}
\format{
A list of named vectors with the correspondence between time-stamped
annotations
}
\source{
This file was created with the script in
"inst/scripts/make-annotation-dictionary.R
}
\description{
Annotations are opinionated and dynamic by nature. They are subjected to
change as more experts look into the data and more evidence is published.
HCATonsilData has been designed to account for that. We timestamp annotations
and plan to include additional annotations if users propose them and we
validate them.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HCATonsilData.R
\name{HCATonsilData}
\alias{HCATonsilData}
\title{Access the Tonsil Atlas data (RNA, ATAC, Multiome, CITE, Spatial)}
\usage{
HCATonsilData(
  assayType = c("RNA", "ATAC", "CITE", "Spatial"),
  cellType = "All",
  version = "2.0",
  processedCounts = TRUE
)
}
\arguments{
\item{assayType}{One of 'RNA', 'ATAC', 'Multiome', 'CITE' or 'Spatial'.}

\item{cellType}{A character vector of length 1 with the desired cell type.
A list of available cell types can be obtained using
\code{listCellTypes(assay_type)}.}

\item{version}{Version of the tonsil atlas data to retrieve: "1.0" (preprint)
or "2.0" (publication, default)}

\item{processedCounts}{Logical scalar. If \code{TRUE}, include the processed
(normalized) counts in addition to the raw counts in the
SingleCellExperiment object.}
}
\value{
A \linkS4class{SingleCellExperiment} object for the cellType
requested. For scATAC-seq, Multiome and CITE we provide the instructions
for downloading the Seurat objects in Zenodo (see vignette)
}
\description{
The data was downloaded from Zenodo
https://zenodo.org/record/8373756
}
\examples{
# retrieve the epithelial scRNA-seq dataset
sce_epithelial <- HCATonsilData(
  assayType = "RNA",
  cellType = "epithelial"
)
sce_epithelial
}
\author{
Ramon Massoni-Badosa
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{colors_20230508}
\alias{colors_20230508}
\title{Color palettes}
\format{
A list with the HEX color codes for each cell type
}
\source{
This file was created with the script in
"inst/scripts/make-annotation-colors.R
}
\description{
Color palettes used in the paper for all cell types
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listCellTypes.R
\name{listCellTypes}
\alias{listCellTypes}
\title{List available cell types for a dataset of the tonsil cell atlas}
\usage{
listCellTypes(
  assayType = c("RNA", "ATAC", "Multiome", "CITE", "Spatial"),
  version = "2.0"
)
}
\arguments{
\item{assayType}{Either 'RNA', 'ATAC', 'Multiome', 'CITE', or 'Spatial'}

\item{version}{Version of the tonsil atlas data: '1.0' (preprint) or '2.0'
or '2.0' (publication, default). Note that, for version 2.0, CD8-T and
ILC-NK are combined in a single 'Cytotoxic' object.}
}
\value{
A character vector with the available cell types for the indicated
dataset.
}
\description{
List available cell types for a dataset of the tonsil cell atlas
}
\examples{
listCellTypes(assayType = "RNA", version = "2.0")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glossary.R
\name{TonsilData_cellinfo_html}
\alias{TonsilData_cellinfo_html}
\title{TonsilData_cellinfo_html}
\usage{
TonsilData_cellinfo_html(
  cellType = NULL,
  display_plot = TRUE,
  output_to = c("single_page", "html_to_embed")
)
}
\arguments{
\item{cellType}{String character, used to define the cell type for which the
information will be displayed. Defaults to \code{NULL} - if left unspecified, the
function returns a list of the possible options}

\item{display_plot}{Logical value, defines whether to include or not a plot
for the UMAP of the selected cell type}

\item{output_to}{Character value, one of "single_page" or "html_to_embed".
Defines in which form the information is returned, either as an individual page
or simply as HTML code to directly embed into other documents.}
}
\value{
Either the HTML code generated to be embedded, or a standalone HTML
page is created - and the location to this temp file is returned as a character
(default).
}
\description{
TonsilData_cellinfo_html
}
\examples{
TonsilData_cellinfo_html("PDC")
TonsilData_cellinfo_html("preB")
TonsilData_cellinfo_html("preT")
TonsilData_cellinfo_html("preT", output_to = "html_to_embed")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glossary.R
\name{TonsilData_cellinfo}
\alias{TonsilData_cellinfo}
\title{TonsilData_cellinfo}
\usage{
TonsilData_cellinfo(cellType = NULL)
}
\arguments{
\item{cellType}{String character, used to define the cell type for which the
information will be displayed. Defaults to \code{NULL} - if left unspecified, the
function returns a list of the possible options}
}
\value{
Invisible \code{NULL} - the information is displayed as a message in the
console.
}
\description{
Query the glossary to traceback the rationale for each annotation of the
tonsil atlas
}
\examples{
TonsilData_cellinfo()

TonsilData_cellinfo("PDC")
}
